#!/usr/bin/python
from __future__ import (absolute_import, division, print_function)
# Copyright 2019 Fortinet, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# the lib use python logging can get it if the following is set in your
# Ansible config.

__metaclass__ = type

ANSIBLE_METADATA = {'status': ['preview'],
                    'supported_by': 'community',
                    'metadata_version': '1.1'}

DOCUMENTATION = '''
---
module: fortios_endpoint_control_profile
short_description: Configure FortiClient endpoint control profiles in Fortinet's FortiOS and FortiGate.
description:
    - This module is able to configure a FortiGate or FortiOS by
      allowing the user to configure endpoint_control feature and profile category.
      Examples includes all options and need to be adjusted to datasources before usage.
      Tested with FOS v6.0.2
version_added: "2.8"
author:
    - Miguel Angel Munoz (@mamunozgonzalez)
    - Nicolas Thomas (@thomnico)
notes:
    - Requires fortiosapi library developed by Fortinet
    - Run as a local_action in your playbook
requirements:
    - fortiosapi>=0.9.8
options:
    host:
       description:
            - FortiOS or FortiGate ip address.
       required: true
    username:
        description:
            - FortiOS or FortiGate username.
        required: true
    password:
        description:
            - FortiOS or FortiGate password.
        default: ""
    vdom:
        description:
            - Virtual domain, among those defined previously. A vdom is a
              virtual instance of the FortiGate that can be configured and
              used as a different unit.
        default: root
    https:
        description:
            - Indicates if the requests towards FortiGate must use HTTPS
              protocol
        type: bool
        default: false
    endpoint_control_profile:
        description:
            - Configure FortiClient endpoint control profiles.
        default: null
        suboptions:
            state:
                description:
                    - Indicates whether to create or remove the object
                choices:
                    - present
                    - absent
            description:
                description:
                    - Description.
            device-groups:
                description:
                    - Device groups.
                suboptions:
                    name:
                        description:
                            - Device group object from available options. Source user.device-group.name user.device-category.name.
                        required: true
            forticlient-android-settings:
                description:
                    - FortiClient settings for Android platform.
                suboptions:
                    disable-wf-when-protected:
                        description:
                            - Enable/disable FortiClient web category filtering when protected by FortiGate.
                        choices:
                            - enable
                            - disable
                    forticlient-advanced-vpn:
                        description:
                            - Enable/disable advanced FortiClient VPN configuration.
                        choices:
                            - enable
                            - disable
                    forticlient-advanced-vpn-buffer:
                        description:
                            - Advanced FortiClient VPN configuration.
                    forticlient-vpn-provisioning:
                        description:
                            - Enable/disable FortiClient VPN provisioning.
                        choices:
                            - enable
                            - disable
                    forticlient-vpn-settings:
                        description:
                            - FortiClient VPN settings.
                        suboptions:
                            auth-method:
                                description:
                                    - Authentication method.
                                choices:
                                    - psk
                                    - certificate
                            name:
                                description:
                                    - VPN name.
                                required: true
                            preshared-key:
                                description:
                                    - Pre-shared secret for PSK authentication.
                            remote-gw:
                                description:
                                    - IP address or FQDN of the remote VPN gateway.
                            sslvpn-access-port:
                                description:
                                    - SSL VPN access port (1 - 65535).
                            sslvpn-require-certificate:
                                description:
                                    - Enable/disable requiring SSL VPN client certificate.
                                choices:
                                    - enable
                                    - disable
                            type:
                                description:
                                    - VPN type (IPsec or SSL VPN).
                                choices:
                                    - ipsec
                                    - ssl
                    forticlient-wf:
                        description:
                            - Enable/disable FortiClient web filtering.
                        choices:
                            - enable
                            - disable
                    forticlient-wf-profile:
                        description:
                            - The FortiClient web filter profile to apply. Source webfilter.profile.name.
            forticlient-ios-settings:
                description:
                    - FortiClient settings for iOS platform.
                suboptions:
                    client-vpn-provisioning:
                        description:
                            - FortiClient VPN provisioning.
                        choices:
                            - enable
                            - disable
                    client-vpn-settings:
                        description:
                            - FortiClient VPN settings.
                        suboptions:
                            auth-method:
                                description:
                                    - Authentication method.
                                choices:
                                    - psk
                                    - certificate
                            name:
                                description:
                                    - VPN name.
                                required: true
                            preshared-key:
                                description:
                                    - Pre-shared secret for PSK authentication.
                            remote-gw:
                                description:
                                    - IP address or FQDN of the remote VPN gateway.
                            sslvpn-access-port:
                                description:
                                    - SSL VPN access port (1 - 65535).
                            sslvpn-require-certificate:
                                description:
                                    - Enable/disable requiring SSL VPN client certificate.
                                choices:
                                    - enable
                                    - disable
                            type:
                                description:
                                    - VPN type (IPsec or SSL VPN).
                                choices:
                                    - ipsec
                                    - ssl
                            vpn-configuration-content:
                                description:
                                    - Content of VPN configuration.
                            vpn-configuration-name:
                                description:
                                    - Name of VPN configuration.
                    configuration-content:
                        description:
                            - Content of configuration profile.
                    configuration-name:
                        description:
                            - Name of configuration profile.
                    disable-wf-when-protected:
                        description:
                            - Enable/disable FortiClient web category filtering when protected by FortiGate.
                        choices:
                            - enable
                            - disable
                    distribute-configuration-profile:
                        description:
                            - Enable/disable configuration profile (.mobileconfig file) distribution.
                        choices:
                            - enable
                            - disable
                    forticlient-wf:
                        description:
                            - Enable/disable FortiClient web filtering.
                        choices:
                            - enable
                            - disable
                    forticlient-wf-profile:
                        description:
                            - The FortiClient web filter profile to apply. Source webfilter.profile.name.
            forticlient-winmac-settings:
                description:
                    - FortiClient settings for Windows/Mac platform.
                suboptions:
                    av-realtime-protection:
                        description:
                            - Enable/disable FortiClient AntiVirus real-time protection.
                        choices:
                            - enable
                            - disable
                    av-signature-up-to-date:
                        description:
                            - Enable/disable FortiClient AV signature updates.
                        choices:
                            - enable
                            - disable
                    forticlient-application-firewall:
                        description:
                            - Enable/disable the FortiClient application firewall.
                        choices:
                            - enable
                            - disable
                    forticlient-application-firewall-list:
                        description:
                            - FortiClient application firewall rule list. Source application.list.name.
                    forticlient-av:
                        description:
                            - Enable/disable FortiClient AntiVirus scanning.
                        choices:
                            - enable
                            - disable
                    forticlient-ems-compliance:
                        description:
                            - Enable/disable FortiClient Enterprise Management Server (EMS) compliance.
                        choices:
                            - enable
                            - disable
                    forticlient-ems-compliance-action:
                        description:
                            - FortiClient EMS compliance action.
                        choices:
                            - block
                            - warning
                    forticlient-ems-entries:
                        description:
                            - FortiClient EMS entries.
                        suboptions:
                            name:
                                description:
                                    - FortiClient EMS name. Source endpoint-control.forticlient-ems.name.
                                required: true
                    forticlient-linux-ver:
                        description:
                            - Minimum FortiClient Linux version.
                    forticlient-log-upload:
                        description:
                            - Enable/disable uploading FortiClient logs.
                        choices:
                            - enable
                            - disable
                    forticlient-log-upload-level:
                        description:
                            - Select the FortiClient logs to upload.
                        choices:
                            - traffic
                            - vulnerability
                            - event
                    forticlient-log-upload-server:
                        description:
                            - IP address or FQDN of the server to which to upload FortiClient logs.
                    forticlient-mac-ver:
                        description:
                            - Minimum FortiClient Mac OS version.
                    forticlient-minimum-software-version:
                        description:
                            - Enable/disable requiring clients to run FortiClient with a minimum software version number.
                        choices:
                            - enable
                            - disable
                    forticlient-operating-system:
                        description:
                            - FortiClient operating system.
                        suboptions:
                            id:
                                description:
                                    - Operating system entry ID.
                                required: true
                            os-name:
                                description:
                                    - "Customize operating system name or Mac OS format:x.x.x"
                            os-type:
                                description:
                                    - Operating system type.
                                choices:
                                    - custom
                                    - mac-os
                                    - win-7
                                    - win-80
                                    - win-81
                                    - win-10
                                    - win-2000
                                    - win-home-svr
                                    - win-svr-10
                                    - win-svr-2003
                                    - win-svr-2003-r2
                                    - win-svr-2008
                                    - win-svr-2008-r2
                                    - win-svr-2012
                                    - win-svr-2012-r2
                                    - win-sto-svr-2003
                                    - win-vista
                                    - win-xp
                                    - ubuntu-linux
                                    - centos-linux
                                    - redhat-linux
                                    - fedora-linux
                    forticlient-own-file:
                        description:
                            - Checking the path and filename of the FortiClient application.
                        suboptions:
                            file:
                                description:
                                    - File path and name.
                            id:
                                description:
                                    - File ID.
                                required: true
                    forticlient-registration-compliance-action:
                        description:
                            - FortiClient registration compliance action.
                        choices:
                            - block
                            - warning
                    forticlient-registry-entry:
                        description:
                            - FortiClient registry entry.
                        suboptions:
                            id:
                                description:
                                    - Registry entry ID.
                                required: true
                            registry-entry:
                                description:
                                    - Registry entry.
                    forticlient-running-app:
                        description:
                            - Use FortiClient to verify if the listed applications are running on the client.
                        suboptions:
                            app-name:
                                description:
                                    - Application name.
                            app-sha256-signature:
                                description:
                                    - App's SHA256 signature.
                            app-sha256-signature2:
                                description:
                                    - App's SHA256 Signature.
                            app-sha256-signature3:
                                description:
                                    - App's SHA256 Signature.
                            app-sha256-signature4:
                                description:
                                    - App's SHA256 Signature.
                            application-check-rule:
                                description:
                                    - Application check rule.
                                choices:
                                    - present
                                    - absent
                            id:
                                description:
                                    - Application ID.
                                required: true
                            process-name:
                                description:
                                    - Process name.
                            process-name2:
                                description:
                                    - Process name.
                            process-name3:
                                description:
                                    - Process name.
                            process-name4:
                                description:
                                    - Process name.
                    forticlient-security-posture:
                        description:
                            - Enable/disable FortiClient security posture check options.
                        choices:
                            - enable
                            - disable
                    forticlient-security-posture-compliance-action:
                        description:
                            - FortiClient security posture compliance action.
                        choices:
                            - block
                            - warning
                    forticlient-system-compliance:
                        description:
                            - Enable/disable enforcement of FortiClient system compliance.
                        choices:
                            - enable
                            - disable
                    forticlient-system-compliance-action:
                        description:
                            - Block or warn clients not compliant with FortiClient requirements.
                        choices:
                            - block
                            - warning
                    forticlient-vuln-scan:
                        description:
                            - Enable/disable FortiClient vulnerability scanning.
                        choices:
                            - enable
                            - disable
                    forticlient-vuln-scan-compliance-action:
                        description:
                            - FortiClient vulnerability compliance action.
                        choices:
                            - block
                            - warning
                    forticlient-vuln-scan-enforce:
                        description:
                            - Configure the level of the vulnerability found that causes a FortiClient vulnerability compliance action.
                        choices:
                            - critical
                            - high
                            - medium
                            - low
                            - info
                    forticlient-vuln-scan-enforce-grace:
                        description:
                            - FortiClient vulnerability scan enforcement grace period (0 - 30 days, default = 1).
                    forticlient-vuln-scan-exempt:
                        description:
                            - Enable/disable compliance exemption for vulnerabilities that cannot be patched automatically.
                        choices:
                            - enable
                            - disable
                    forticlient-wf:
                        description:
                            - Enable/disable FortiClient web filtering.
                        choices:
                            - enable
                            - disable
                    forticlient-wf-profile:
                        description:
                            - The FortiClient web filter profile to apply. Source webfilter.profile.name.
                    forticlient-win-ver:
                        description:
                            - Minimum FortiClient Windows version.
                    os-av-software-installed:
                        description:
                            - Enable/disable checking for OS recognized AntiVirus software.
                        choices:
                            - enable
                            - disable
                    sandbox-address:
                        description:
                            - FortiSandbox address.
                    sandbox-analysis:
                        description:
                            - Enable/disable sending files to FortiSandbox for analysis.
                        choices:
                            - enable
                            - disable
            on-net-addr:
                description:
                    - Addresses for on-net detection.
                suboptions:
                    name:
                        description:
                            - Address object from available options. Source firewall.address.name firewall.addrgrp.name.
                        required: true
            profile-name:
                description:
                    - Profile name.
                required: true
            replacemsg-override-group:
                description:
                    - Select an endpoint control replacement message override group from available options. Source system.replacemsg-group.name.
            src-addr:
                description:
                    - Source addresses.
                suboptions:
                    name:
                        description:
                            - Address object from available options. Source firewall.address.name firewall.addrgrp.name.
                        required: true
            user-groups:
                description:
                    - User groups.
                suboptions:
                    name:
                        description:
                            - User group name. Source user.group.name.
                        required: true
            users:
                description:
                    - Users.
                suboptions:
                    name:
                        description:
                            - User name. Source user.local.name.
                        required: true
'''

EXAMPLES = '''
- hosts: localhost
  vars:
   host: "192.168.122.40"
   username: "admin"
   password: ""
   vdom: "root"
  tasks:
  - name: Configure FortiClient endpoint control profiles.
    fortios_endpoint_control_profile:
      host:  "{{ host }}"
      username: "{{ username }}"
      password: "{{ password }}"
      vdom:  "{{ vdom }}"
      endpoint_control_profile:
        state: "present"
        description: "<your_own_value>"
        device-groups:
         -
            name: "default_name_5 (source user.device-group.name user.device-category.name)"
        forticlient-android-settings:
            disable-wf-when-protected: "enable"
            forticlient-advanced-vpn: "enable"
            forticlient-advanced-vpn-buffer: "<your_own_value>"
            forticlient-vpn-provisioning: "enable"
            forticlient-vpn-settings:
             -
                auth-method: "psk"
                name: "default_name_13"
                preshared-key: "<your_own_value>"
                remote-gw: "<your_own_value>"
                sslvpn-access-port: "16"
                sslvpn-require-certificate: "enable"
                type: "ipsec"
            forticlient-wf: "enable"
            forticlient-wf-profile: "<your_own_value> (source webfilter.profile.name)"
        forticlient-ios-settings:
            client-vpn-provisioning: "enable"
            client-vpn-settings:
             -
                auth-method: "psk"
                name: "default_name_25"
                preshared-key: "<your_own_value>"
                remote-gw: "<your_own_value>"
                sslvpn-access-port: "28"
                sslvpn-require-certificate: "enable"
                type: "ipsec"
                vpn-configuration-content: "<your_own_value>"
                vpn-configuration-name: "<your_own_value>"
            configuration-content: "<your_own_value>"
            configuration-name: "<your_own_value>"
            disable-wf-when-protected: "enable"
            distribute-configuration-profile: "enable"
            forticlient-wf: "enable"
            forticlient-wf-profile: "<your_own_value> (source webfilter.profile.name)"
        forticlient-winmac-settings:
            av-realtime-protection: "enable"
            av-signature-up-to-date: "enable"
            forticlient-application-firewall: "enable"
            forticlient-application-firewall-list: "<your_own_value> (source application.list.name)"
            forticlient-av: "enable"
            forticlient-ems-compliance: "enable"
            forticlient-ems-compliance-action: "block"
            forticlient-ems-entries:
             -
                name: "default_name_48 (source endpoint-control.forticlient-ems.name)"
            forticlient-linux-ver: "<your_own_value>"
            forticlient-log-upload: "enable"
            forticlient-log-upload-level: "traffic"
            forticlient-log-upload-server: "<your_own_value>"
            forticlient-mac-ver: "<your_own_value>"
            forticlient-minimum-software-version: "enable"
            forticlient-operating-system:
             -
                id:  "56"
                os-name: "<your_own_value>"
                os-type: "custom"
            forticlient-own-file:
             -
                file: "<your_own_value>"
                id:  "61"
            forticlient-registration-compliance-action: "block"
            forticlient-registry-entry:
             -
                id:  "64"
                registry-entry: "<your_own_value>"
            forticlient-running-app:
             -
                app-name: "<your_own_value>"
                app-sha256-signature: "<your_own_value>"
                app-sha256-signature2: "<your_own_value>"
                app-sha256-signature3: "<your_own_value>"
                app-sha256-signature4: "<your_own_value>"
                application-check-rule: "present"
                id:  "73"
                process-name: "<your_own_value>"
                process-name2: "<your_own_value>"
                process-name3: "<your_own_value>"
                process-name4: "<your_own_value>"
            forticlient-security-posture: "enable"
            forticlient-security-posture-compliance-action: "block"
            forticlient-system-compliance: "enable"
            forticlient-system-compliance-action: "block"
            forticlient-vuln-scan: "enable"
            forticlient-vuln-scan-compliance-action: "block"
            forticlient-vuln-scan-enforce: "critical"
            forticlient-vuln-scan-enforce-grace: "85"
            forticlient-vuln-scan-exempt: "enable"
            forticlient-wf: "enable"
            forticlient-wf-profile: "<your_own_value> (source webfilter.profile.name)"
            forticlient-win-ver: "<your_own_value>"
            os-av-software-installed: "enable"
            sandbox-address: "<your_own_value>"
            sandbox-analysis: "enable"
        on-net-addr:
         -
            name: "default_name_94 (source firewall.address.name firewall.addrgrp.name)"
        profile-name: "<your_own_value>"
        replacemsg-override-group: "<your_own_value> (source system.replacemsg-group.name)"
        src-addr:
         -
            name: "default_name_98 (source firewall.address.name firewall.addrgrp.name)"
        user-groups:
         -
            name: "default_name_100 (source user.group.name)"
        users:
         -
            name: "default_name_102 (source user.local.name)"
'''

RETURN = '''
build:
  description: Build number of the fortigate image
  returned: always
  type: str
  sample: '1547'
http_method:
  description: Last method used to provision the content into FortiGate
  returned: always
  type: str
  sample: 'PUT'
http_status:
  description: Last result given by FortiGate on last operation applied
  returned: always
  type: str
  sample: "200"
mkey:
  description: Master key (id) used in the last call to FortiGate
  returned: success
  type: str
  sample: "id"
name:
  description: Name of the table used to fulfill the request
  returned: always
  type: str
  sample: "urlfilter"
path:
  description: Path of the table used to fulfill the request
  returned: always
  type: str
  sample: "webfilter"
revision:
  description: Internal revision number
  returned: always
  type: str
  sample: "17.0.2.10658"
serial:
  description: Serial number of the unit
  returned: always
  type: str
  sample: "FGVMEVYYQT3AB5352"
status:
  description: Indication of the operation's result
  returned: always
  type: str
  sample: "success"
vdom:
  description: Virtual domain used
  returned: always
  type: str
  sample: "root"
version:
  description: Version of the FortiGate
  returned: always
  type: str
  sample: "v5.6.3"

'''

from ansible.module_utils.basic import AnsibleModule

fos = None


def login(data):
    host = data['host']
    username = data['username']
    password = data['password']

    fos.debug('on')
    if 'https' in data and not data['https']:
        fos.https('off')
    else:
        fos.https('on')

    fos.login(host, username, password)


def filter_endpoint_control_profile_data(json):
    option_list = ['description', 'device-groups', 'forticlient-android-settings',
                   'forticlient-ios-settings', 'forticlient-winmac-settings', 'on-net-addr',
                   'profile-name', 'replacemsg-override-group', 'src-addr',
                   'user-groups', 'users']
    dictionary = {}

    for attribute in option_list:
        if attribute in json and json[attribute] is not None:
            dictionary[attribute] = json[attribute]

    return dictionary


def endpoint_control_profile(data, fos):
    vdom = data['vdom']
    endpoint_control_profile_data = data['endpoint_control_profile']
    filtered_data = filter_endpoint_control_profile_data(endpoint_control_profile_data)
    if endpoint_control_profile_data['state'] == "present":
        return fos.set('endpoint-control',
                       'profile',
                       data=filtered_data,
                       vdom=vdom)

    elif endpoint_control_profile_data['state'] == "absent":
        return fos.delete('endpoint-control',
                          'profile',
                          mkey=filtered_data['profile-name'],
                          vdom=vdom)


def fortios_endpoint_control(data, fos):
    login(data)

    methodlist = ['endpoint_control_profile']
    for method in methodlist:
        if data[method]:
            resp = eval(method)(data, fos)
            break

    fos.logout()
    return not resp['status'] == "success", resp['status'] == "success", resp


def main():
    fields = {
        "host": {"required": True, "type": "str"},
        "username": {"required": True, "type": "str"},
        "password": {"required": False, "type": "str", "no_log": True},
        "vdom": {"required": False, "type": "str", "default": "root"},
        "https": {"required": False, "type": "bool", "default": "False"},
        "endpoint_control_profile": {
            "required": False, "type": "dict",
            "options": {
                "state": {"required": True, "type": "str",
                          "choices": ["present", "absent"]},
                "description": {"required": False, "type": "str"},
                "device-groups": {"required": False, "type": "list",
                                  "options": {
                                      "name": {"required": True, "type": "str"}
                                  }},
                "forticlient-android-settings": {"required": False, "type": "dict",
                                                 "options": {
                                                     "disable-wf-when-protected": {"required": False, "type": "str",
                                                                                   "choices": ["enable", "disable"]},
                                                     "forticlient-advanced-vpn": {"required": False, "type": "str",
                                                                                  "choices": ["enable", "disable"]},
                                                     "forticlient-advanced-vpn-buffer": {"required": False, "type": "str"},
                                                     "forticlient-vpn-provisioning": {"required": False, "type": "str",
                                                                                      "choices": ["enable", "disable"]},
                                                     "forticlient-vpn-settings": {"required": False, "type": "list",
                                                                                  "options": {
                                                                                      "auth-method": {"required": False, "type": "str",
                                                                                                      "choices": ["psk", "certificate"]},
                                                                                      "name": {"required": True, "type": "str"},
                                                                                      "preshared-key": {"required": False, "type": "str"},
                                                                                      "remote-gw": {"required": False, "type": "str"},
                                                                                      "sslvpn-access-port": {"required": False, "type": "int"},
                                                                                      "sslvpn-require-certificate": {"required": False, "type": "str",
                                                                                                                     "choices": ["enable", "disable"]},
                                                                                      "type": {"required": False, "type": "str",
                                                                                               "choices": ["ipsec", "ssl"]}
                                                                                  }},
                                                     "forticlient-wf": {"required": False, "type": "str",
                                                                        "choices": ["enable", "disable"]},
                                                     "forticlient-wf-profile": {"required": False, "type": "str"}
                                                 }},
                "forticlient-ios-settings": {"required": False, "type": "dict",
                                             "options": {
                                                 "client-vpn-provisioning": {"required": False, "type": "str",
                                                                             "choices": ["enable", "disable"]},
                                                 "client-vpn-settings": {"required": False, "type": "list",
                                                                         "options": {
                                                                             "auth-method": {"required": False, "type": "str",
                                                                                             "choices": ["psk", "certificate"]},
                                                                             "name": {"required": True, "type": "str"},
                                                                             "preshared-key": {"required": False, "type": "str"},
                                                                             "remote-gw": {"required": False, "type": "str"},
                                                                             "sslvpn-access-port": {"required": False, "type": "int"},
                                                                             "sslvpn-require-certificate": {"required": False, "type": "str",
                                                                                                            "choices": ["enable", "disable"]},
                                                                             "type": {"required": False, "type": "str",
                                                                                      "choices": ["ipsec", "ssl"]},
                                                                             "vpn-configuration-content": {"required": False, "type": "str"},
                                                                             "vpn-configuration-name": {"required": False, "type": "str"}
                                                                         }},
                                                 "configuration-content": {"required": False, "type": "str"},
                                                 "configuration-name": {"required": False, "type": "str"},
                                                 "disable-wf-when-protected": {"required": False, "type": "str",
                                                                               "choices": ["enable", "disable"]},
                                                 "distribute-configuration-profile": {"required": False, "type": "str",
                                                                                      "choices": ["enable", "disable"]},
                                                 "forticlient-wf": {"required": False, "type": "str",
                                                                    "choices": ["enable", "disable"]},
                                                 "forticlient-wf-profile": {"required": False, "type": "str"}
                                             }},
                "forticlient-winmac-settings": {"required": False, "type": "dict",
                                                "options": {
                                                    "av-realtime-protection": {"required": False, "type": "str",
                                                                               "choices": ["enable", "disable"]},
                                                    "av-signature-up-to-date": {"required": False, "type": "str",
                                                                                "choices": ["enable", "disable"]},
                                                    "forticlient-application-firewall": {"required": False, "type": "str",
                                                                                         "choices": ["enable", "disable"]},
                                                    "forticlient-application-firewall-list": {"required": False, "type": "str"},
                                                    "forticlient-av": {"required": False, "type": "str",
                                                                       "choices": ["enable", "disable"]},
                                                    "forticlient-ems-compliance": {"required": False, "type": "str",
                                                                                   "choices": ["enable", "disable"]},
                                                    "forticlient-ems-compliance-action": {"required": False, "type": "str",
                                                                                          "choices": ["block", "warning"]},
                                                    "forticlient-ems-entries": {"required": False, "type": "list",
                                                                                "options": {
                                                                                    "name": {"required": True, "type": "str"}
                                                                                }},
                                                    "forticlient-linux-ver": {"required": False, "type": "str"},
                                                    "forticlient-log-upload": {"required": False, "type": "str",
                                                                               "choices": ["enable", "disable"]},
                                                    "forticlient-log-upload-level": {"required": False, "type": "str",
                                                                                     "choices": ["traffic", "vulnerability", "event"]},
                                                    "forticlient-log-upload-server": {"required": False, "type": "str"},
                                                    "forticlient-mac-ver": {"required": False, "type": "str"},
                                                    "forticlient-minimum-software-version": {"required": False, "type": "str",
                                                                                             "choices": ["enable", "disable"]},
                                                    "forticlient-operating-system": {"required": False, "type": "list",
                                                                                     "options": {
                                                                                         "id": {"required": True, "type": "int"},
                                                                                         "os-name": {"required": False, "type": "str"},
                                                                                         "os-type": {"required": False, "type": "str",
                                                                                                     "choices": ["custom", "mac-os", "win-7",
                                                                                                                 "win-80", "win-81", "win-10",
                                                                                                                 "win-2000", "win-home-svr", "win-svr-10",
                                                                                                                 "win-svr-2003", "win-svr-2003-r2",
                                                                                                                 "win-svr-2008", "win-svr-2008-r2",
                                                                                                                 "win-svr-2012", "win-svr-2012-r2",
                                                                                                                 "win-sto-svr-2003", "win-vista", "win-xp",
                                                                                                                 "ubuntu-linux", "centos-linux", "redhat-linux",
                                                                                                                 "fedora-linux"]}
                                                                                     }},
                                                    "forticlient-own-file": {"required": False, "type": "list",
                                                                             "options": {
                                                                                 "file": {"required": False, "type": "str"},
                                                                                 "id": {"required": True, "type": "int"}
                                                                             }},
                                                    "forticlient-registration-compliance-action": {"required": False, "type": "str",
                                                                                                   "choices": ["block", "warning"]},
                                                    "forticlient-registry-entry": {"required": False, "type": "list",
                                                                                   "options": {
                                                                                       "id": {"required": True, "type": "int"},
                                                                                       "registry-entry": {"required": False, "type": "str"}
                                                                                   }},
                                                    "forticlient-running-app": {"required": False, "type": "list",
                                                                                "options": {
                                                                                    "app-name": {"required": False, "type": "str"},
                                                                                    "app-sha256-signature": {"required": False, "type": "str"},
                                                                                    "app-sha256-signature2": {"required": False, "type": "str"},
                                                                                    "app-sha256-signature3": {"required": False, "type": "str"},
                                                                                    "app-sha256-signature4": {"required": False, "type": "str"},
                                                                                    "application-check-rule": {"required": False, "type": "str",
                                                                                                               "choices": ["present", "absent"]},
                                                                                    "id": {"required": True, "type": "int"},
                                                                                    "process-name": {"required": False, "type": "str"},
                                                                                    "process-name2": {"required": False, "type": "str"},
                                                                                    "process-name3": {"required": False, "type": "str"},
                                                                                    "process-name4": {"required": False, "type": "str"}
                                                                                }},
                                                    "forticlient-security-posture": {"required": False, "type": "str",
                                                                                     "choices": ["enable", "disable"]},
                                                    "forticlient-security-posture-compliance-action": {"required": False, "type": "str",
                                                                                                       "choices": ["block", "warning"]},
                                                    "forticlient-system-compliance": {"required": False, "type": "str",
                                                                                      "choices": ["enable", "disable"]},
                                                    "forticlient-system-compliance-action": {"required": False, "type": "str",
                                                                                             "choices": ["block", "warning"]},
                                                    "forticlient-vuln-scan": {"required": False, "type": "str",
                                                                              "choices": ["enable", "disable"]},
                                                    "forticlient-vuln-scan-compliance-action": {"required": False, "type": "str",
                                                                                                "choices": ["block", "warning"]},
                                                    "forticlient-vuln-scan-enforce": {"required": False, "type": "str",
                                                                                      "choices": ["critical", "high", "medium",
                                                                                                  "low", "info"]},
                                                    "forticlient-vuln-scan-enforce-grace": {"required": False, "type": "int"},
                                                    "forticlient-vuln-scan-exempt": {"required": False, "type": "str",
                                                                                     "choices": ["enable", "disable"]},
                                                    "forticlient-wf": {"required": False, "type": "str",
                                                                       "choices": ["enable", "disable"]},
                                                    "forticlient-wf-profile": {"required": False, "type": "str"},
                                                    "forticlient-win-ver": {"required": False, "type": "str"},
                                                    "os-av-software-installed": {"required": False, "type": "str",
                                                                                 "choices": ["enable", "disable"]},
                                                    "sandbox-address": {"required": False, "type": "str"},
                                                    "sandbox-analysis": {"required": False, "type": "str",
                                                                         "choices": ["enable", "disable"]}
                                                }},
                "on-net-addr": {"required": False, "type": "list",
                                "options": {
                                    "name": {"required": True, "type": "str"}
                                }},
                "profile-name": {"required": True, "type": "str"},
                "replacemsg-override-group": {"required": False, "type": "str"},
                "src-addr": {"required": False, "type": "list",
                             "options": {
                                 "name": {"required": True, "type": "str"}
                             }},
                "user-groups": {"required": False, "type": "list",
                                "options": {
                                    "name": {"required": True, "type": "str"}
                                }},
                "users": {"required": False, "type": "list",
                          "options": {
                              "name": {"required": True, "type": "str"}
                          }}

            }
        }
    }

    module = AnsibleModule(argument_spec=fields,
                           supports_check_mode=False)
    try:
        from fortiosapi import FortiOSAPI
    except ImportError:
        module.fail_json(msg="fortiosapi module is required")

    global fos
    fos = FortiOSAPI()

    is_error, has_changed, result = fortios_endpoint_control(module.params, fos)

    if not is_error:
        module.exit_json(changed=has_changed, meta=result)
    else:
        module.fail_json(msg="Error in repo", meta=result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
from __future__ import (absolute_import, division, print_function)
# Copyright 2019 Fortinet, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# the lib use python logging can get it if the following is set in your
# Ansible config.

__metaclass__ = type

ANSIBLE_METADATA = {'status': ['preview'],
                    'supported_by': 'community',
                    'metadata_version': '1.1'}

DOCUMENTATION = '''
---
module: fortios_firewall_sniffer
short_description: Configure sniffer in Fortinet's FortiOS and FortiGate.
description:
    - This module is able to configure a FortiGate or FortiOS by
      allowing the user to configure firewall feature and sniffer category.
      Examples includes all options and need to be adjusted to datasources before usage.
      Tested with FOS v6.0.2
version_added: "2.8"
author:
    - Miguel Angel Munoz (@mamunozgonzalez)
    - Nicolas Thomas (@thomnico)
notes:
    - Requires fortiosapi library developed by Fortinet
    - Run as a local_action in your playbook
requirements:
    - fortiosapi>=0.9.8
options:
    host:
       description:
            - FortiOS or FortiGate ip adress.
       required: true
    username:
        description:
            - FortiOS or FortiGate username.
        required: true
    password:
        description:
            - FortiOS or FortiGate password.
        default: ""
    vdom:
        description:
            - Virtual domain, among those defined previously. A vdom is a
              virtual instance of the FortiGate that can be configured and
              used as a different unit.
        default: root
    https:
        description:
            - Indicates if the requests towards FortiGate must use HTTPS
              protocol
        type: bool
        default: true
    firewall_sniffer:
        description:
            - Configure sniffer.
        default: null
        suboptions:
            state:
                description:
                    - Indicates whether to create or remove the object
                choices:
                    - present
                    - absent
            anomaly:
                description:
                    - Configuration method to edit Denial of Service (DoS) anomaly settings.
                suboptions:
                    action:
                        description:
                            - Action taken when the threshold is reached.
                        choices:
                            - pass
                            - block
                    log:
                        description:
                            - Enable/disable anomaly logging.
                        choices:
                            - enable
                            - disable
                    name:
                        description:
                            - Anomaly name.
                        required: true
                    quarantine:
                        description:
                            - Quarantine method.
                        choices:
                            - none
                            - attacker
                    quarantine-expiry:
                        description:
                            - Duration of quarantine. (Format ###d##h##m, minimum 1m, maximum 364d23h59m, default = 5m). Requires quarantine set to attacker.
                    quarantine-log:
                        description:
                            - Enable/disable quarantine logging.
                        choices:
                            - disable
                            - enable
                    status:
                        description:
                            - Enable/disable this anomaly.
                        choices:
                            - disable
                            - enable
                    threshold:
                        description:
                            - Anomaly threshold. Number of detected instances per minute that triggers the anomaly action.
                    threshold(default):
                        description:
                            - Number of detected instances per minute which triggers action (1 - 2147483647, default = 1000). Note that each anomaly has a
                               different threshold value assigned to it.
            application-list:
                description:
                    - Name of an existing application list. Source application.list.name.
            application-list-status:
                description:
                    - Enable/disable application control profile.
                choices:
                    - enable
                    - disable
            av-profile:
                description:
                    - Name of an existing antivirus profile. Source antivirus.profile.name.
            av-profile-status:
                description:
                    - Enable/disable antivirus profile.
                choices:
                    - enable
                    - disable
            dlp-sensor:
                description:
                    - Name of an existing DLP sensor. Source dlp.sensor.name.
            dlp-sensor-status:
                description:
                    - Enable/disable DLP sensor.
                choices:
                    - enable
                    - disable
            dsri:
                description:
                    - Enable/disable DSRI.
                choices:
                    - enable
                    - disable
            host:
                description:
                    - "Hosts to filter for in sniffer traffic (Format examples: 1.1.1.1, 2.2.2.0/24, 3.3.3.3/255.255.255.0, 4.4.4.0-4.4.4.240)."
            id:
                description:
                    - Sniffer ID.
                required: true
            interface:
                description:
                    - Interface name that traffic sniffing will take place on. Source system.interface.name.
            ips-dos-status:
                description:
                    - Enable/disable IPS DoS anomaly detection.
                choices:
                    - enable
                    - disable
            ips-sensor:
                description:
                    - Name of an existing IPS sensor. Source ips.sensor.name.
            ips-sensor-status:
                description:
                    - Enable/disable IPS sensor.
                choices:
                    - enable
                    - disable
            ipv6:
                description:
                    - Enable/disable sniffing IPv6 packets.
                choices:
                    - enable
                    - disable
            logtraffic:
                description:
                    - Either log all sessions, only sessions that have a security profile applied, or disable all logging for this policy.
                choices:
                    - all
                    - utm
                    - disable
            max-packet-count:
                description:
                    - Maximum packet count (1 - 1000000, default = 10000).
            non-ip:
                description:
                    - Enable/disable sniffing non-IP packets.
                choices:
                    - enable
                    - disable
            port:
                description:
                    - "Ports to sniff (Format examples: 10, :20, 30:40, 50-, 100-200)."
            protocol:
                description:
                    - Integer value for the protocol type as defined by IANA (0 - 255).
            scan-botnet-connections:
                description:
                    - Enable/disable scanning of connections to Botnet servers.
                choices:
                    - disable
                    - block
                    - monitor
            spamfilter-profile:
                description:
                    - Name of an existing spam filter profile. Source spamfilter.profile.name.
            spamfilter-profile-status:
                description:
                    - Enable/disable spam filter.
                choices:
                    - enable
                    - disable
            status:
                description:
                    - Enable/disable the active status of the sniffer.
                choices:
                    - enable
                    - disable
            vlan:
                description:
                    - List of VLANs to sniff.
            webfilter-profile:
                description:
                    - Name of an existing web filter profile. Source webfilter.profile.name.
            webfilter-profile-status:
                description:
                    - Enable/disable web filter profile.
                choices:
                    - enable
                    - disable
'''

EXAMPLES = '''
- hosts: localhost
  vars:
   host: "192.168.122.40"
   username: "admin"
   password: ""
   vdom: "root"
  tasks:
  - name: Configure sniffer.
    fortios_firewall_sniffer:
      host:  "{{ host }}"
      username: "{{ username }}"
      password: "{{ password }}"
      vdom:  "{{ vdom }}"
      https: "False"
      firewall_sniffer:
        state: "present"
        anomaly:
         -
            action: "pass"
            log: "enable"
            name: "default_name_6"
            quarantine: "none"
            quarantine-expiry: "<your_own_value>"
            quarantine-log: "disable"
            status: "disable"
            threshold: "11"
            threshold(default): "12"
        application-list: "<your_own_value> (source application.list.name)"
        application-list-status: "enable"
        av-profile: "<your_own_value> (source antivirus.profile.name)"
        av-profile-status: "enable"
        dlp-sensor: "<your_own_value> (source dlp.sensor.name)"
        dlp-sensor-status: "enable"
        dsri: "enable"
        host: "myhostname"
        id:  "21"
        interface: "<your_own_value> (source system.interface.name)"
        ips-dos-status: "enable"
        ips-sensor: "<your_own_value> (source ips.sensor.name)"
        ips-sensor-status: "enable"
        ipv6: "enable"
        logtraffic: "all"
        max-packet-count: "28"
        non-ip: "enable"
        port: "<your_own_value>"
        protocol: "<your_own_value>"
        scan-botnet-connections: "disable"
        spamfilter-profile: "<your_own_value> (source spamfilter.profile.name)"
        spamfilter-profile-status: "enable"
        status: "enable"
        vlan: "<your_own_value>"
        webfilter-profile: "<your_own_value> (source webfilter.profile.name)"
        webfilter-profile-status: "enable"
'''

RETURN = '''
build:
  description: Build number of the fortigate image
  returned: always
  type: str
  sample: '1547'
http_method:
  description: Last method used to provision the content into FortiGate
  returned: always
  type: str
  sample: 'PUT'
http_status:
  description: Last result given by FortiGate on last operation applied
  returned: always
  type: str
  sample: "200"
mkey:
  description: Master key (id) used in the last call to FortiGate
  returned: success
  type: str
  sample: "id"
name:
  description: Name of the table used to fulfill the request
  returned: always
  type: str
  sample: "urlfilter"
path:
  description: Path of the table used to fulfill the request
  returned: always
  type: str
  sample: "webfilter"
revision:
  description: Internal revision number
  returned: always
  type: str
  sample: "17.0.2.10658"
serial:
  description: Serial number of the unit
  returned: always
  type: str
  sample: "FGVMEVYYQT3AB5352"
status:
  description: Indication of the operation's result
  returned: always
  type: str
  sample: "success"
vdom:
  description: Virtual domain used
  returned: always
  type: str
  sample: "root"
version:
  description: Version of the FortiGate
  returned: always
  type: str
  sample: "v5.6.3"

'''

from ansible.module_utils.basic import AnsibleModule

fos = None


def login(data):
    host = data['host']
    username = data['username']
    password = data['password']

    fos.debug('on')
    if 'https' in data and not data['https']:
        fos.https('off')
    else:
        fos.https('on')

    fos.login(host, username, password)


def filter_firewall_sniffer_data(json):
    option_list = ['anomaly', 'application-list', 'application-list-status',
                   'av-profile', 'av-profile-status', 'dlp-sensor',
                   'dlp-sensor-status', 'dsri', 'host',
                   'id', 'interface', 'ips-dos-status',
                   'ips-sensor', 'ips-sensor-status', 'ipv6',
                   'logtraffic', 'max-packet-count', 'non-ip',
                   'port', 'protocol', 'scan-botnet-connections',
                   'spamfilter-profile', 'spamfilter-profile-status', 'status',
                   'vlan', 'webfilter-profile', 'webfilter-profile-status']
    dictionary = {}

    for attribute in option_list:
        if attribute in json and json[attribute] is not None:
            dictionary[attribute] = json[attribute]

    return dictionary


def firewall_sniffer(data, fos):
    vdom = data['vdom']
    firewall_sniffer_data = data['firewall_sniffer']
    filtered_data = filter_firewall_sniffer_data(firewall_sniffer_data)
    if firewall_sniffer_data['state'] == "present":
        return fos.set('firewall',
                       'sniffer',
                       data=filtered_data,
                       vdom=vdom)

    elif firewall_sniffer_data['state'] == "absent":
        return fos.delete('firewall',
                          'sniffer',
                          mkey=filtered_data['id'],
                          vdom=vdom)


def fortios_firewall(data, fos):
    login(data)

    methodlist = ['firewall_sniffer']
    for method in methodlist:
        if data[method]:
            resp = eval(method)(data, fos)
            break

    fos.logout()
    return not resp['status'] == "success", resp['status'] == "success", resp


def main():
    fields = {
        "host": {"required": True, "type": "str"},
        "username": {"required": True, "type": "str"},
        "password": {"required": False, "type": "str", "no_log": True},
        "vdom": {"required": False, "type": "str", "default": "root"},
        "https": {"required": False, "type": "bool", "default": True},
        "firewall_sniffer": {
            "required": False, "type": "dict",
            "options": {
                "state": {"required": True, "type": "str",
                          "choices": ["present", "absent"]},
                "anomaly": {"required": False, "type": "list",
                            "options": {
                                "action": {"required": False, "type": "str",
                                           "choices": ["pass", "block"]},
                                "log": {"required": False, "type": "str",
                                        "choices": ["enable", "disable"]},
                                "name": {"required": True, "type": "str"},
                                "quarantine": {"required": False, "type": "str",
                                               "choices": ["none", "attacker"]},
                                "quarantine-expiry": {"required": False, "type": "str"},
                                "quarantine-log": {"required": False, "type": "str",
                                                   "choices": ["disable", "enable"]},
                                "status": {"required": False, "type": "str",
                                           "choices": ["disable", "enable"]},
                                "threshold": {"required": False, "type": "int"},
                                "threshold(default)": {"required": False, "type": "int"}
                            }},
                "application-list": {"required": False, "type": "str"},
                "application-list-status": {"required": False, "type": "str",
                                            "choices": ["enable", "disable"]},
                "av-profile": {"required": False, "type": "str"},
                "av-profile-status": {"required": False, "type": "str",
                                      "choices": ["enable", "disable"]},
                "dlp-sensor": {"required": False, "type": "str"},
                "dlp-sensor-status": {"required": False, "type": "str",
                                      "choices": ["enable", "disable"]},
                "dsri": {"required": False, "type": "str",
                         "choices": ["enable", "disable"]},
                "host": {"required": False, "type": "str"},
                "id": {"required": True, "type": "int"},
                "interface": {"required": False, "type": "str"},
                "ips-dos-status": {"required": False, "type": "str",
                                   "choices": ["enable", "disable"]},
                "ips-sensor": {"required": False, "type": "str"},
                "ips-sensor-status": {"required": False, "type": "str",
                                      "choices": ["enable", "disable"]},
                "ipv6": {"required": False, "type": "str",
                         "choices": ["enable", "disable"]},
                "logtraffic": {"required": False, "type": "str",
                               "choices": ["all", "utm", "disable"]},
                "max-packet-count": {"required": False, "type": "int"},
                "non-ip": {"required": False, "type": "str",
                           "choices": ["enable", "disable"]},
                "port": {"required": False, "type": "str"},
                "protocol": {"required": False, "type": "str"},
                "scan-botnet-connections": {"required": False, "type": "str",
                                            "choices": ["disable", "block", "monitor"]},
                "spamfilter-profile": {"required": False, "type": "str"},
                "spamfilter-profile-status": {"required": False, "type": "str",
                                              "choices": ["enable", "disable"]},
                "status": {"required": False, "type": "str",
                           "choices": ["enable", "disable"]},
                "vlan": {"required": False, "type": "str"},
                "webfilter-profile": {"required": False, "type": "str"},
                "webfilter-profile-status": {"required": False, "type": "str",
                                             "choices": ["enable", "disable"]}

            }
        }
    }

    module = AnsibleModule(argument_spec=fields,
                           supports_check_mode=False)
    try:
        from fortiosapi import FortiOSAPI
    except ImportError:
        module.fail_json(msg="fortiosapi module is required")

    global fos
    fos = FortiOSAPI()

    is_error, has_changed, result = fortios_firewall(module.params, fos)

    if not is_error:
        module.exit_json(changed=has_changed, meta=result)
    else:
        module.fail_json(msg="Error in repo", meta=result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright: (c) 2017, F5 Networks Inc.
# GNU General Public License v3.0 (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['stableinterface'],
                    'supported_by': 'certified'}

DOCUMENTATION = r'''
---
module: bigip_configsync_action
short_description: Perform different actions related to config-sync
description:
  - Allows one to run different config-sync actions. These actions allow
    you to manually sync your configuration across multiple BIG-IPs when
    those devices are in an HA pair.
version_added: 2.4
options:
  device_group:
    description:
      - The device group that you want to perform config-sync actions on.
    type: str
    required: True
  sync_device_to_group:
    description:
      - Specifies that the system synchronizes configuration data from this
        device to other members of the device group. In this case, the device
        will do a "push" to all the other devices in the group. This option
        is mutually exclusive with the C(sync_group_to_device) option.
    type: bool
  sync_most_recent_to_device:
    description:
      - Specifies that the system synchronizes configuration data from the
        device with the most recent configuration. In this case, the device
        will do a "pull" from the most recently updated device. This option
        is mutually exclusive with the C(sync_device_to_group) options.
    type: bool
  overwrite_config:
    description:
      - Indicates that the sync operation overwrites the configuration on
        the target.
    type: bool
    default: no
notes:
  - Requires the objectpath Python package on the host. This is as easy as
    C(pip install objectpath).
extends_documentation_fragment: f5
author:
  - Tim Rupp (@caphrim007)
  - Wojciech Wypior (@wojtek0806)
'''

EXAMPLES = r'''
- name: Sync configuration from device to group
  bigip_configsync_action:
    device_group: foo-group
    sync_device_to_group: yes
    provider:
      server: lb.mydomain.com
      user: admin
      password: secret
  delegate_to: localhost

- name: Sync configuration from most recent device to the current host
  bigip_configsync_action:
    device_group: foo-group
    sync_most_recent_to_device: yes
    provider:
      server: lb.mydomain.com
      user: admin
      password: secret
  delegate_to: localhost

- name: Perform an initial sync of a device to a new device group
  bigip_configsync_action:
    device_group: new-device-group
    sync_device_to_group: yes
    provider:
      server: lb.mydomain.com
      user: admin
      password: secret
  delegate_to: localhost
'''

RETURN = r'''
# only common fields returned
'''

import re
import time

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.parsing.convert_bool import BOOLEANS_TRUE

try:
    from library.module_utils.network.f5.bigip import F5RestClient
    from library.module_utils.network.f5.common import F5ModuleError
    from library.module_utils.network.f5.common import AnsibleF5Parameters
    from library.module_utils.network.f5.common import f5_argument_spec
except ImportError:
    from ansible.module_utils.network.f5.bigip import F5RestClient
    from ansible.module_utils.network.f5.common import F5ModuleError
    from ansible.module_utils.network.f5.common import AnsibleF5Parameters
    from ansible.module_utils.network.f5.common import f5_argument_spec

try:
    from objectpath import Tree
    HAS_OBJPATH = True
except ImportError:
    HAS_OBJPATH = False


class Parameters(AnsibleF5Parameters):
    api_attributes = []
    returnables = []


class ApiParameters(Parameters):
    pass


class ModuleParameters(Parameters):
    @property
    def direction(self):
        if self.sync_device_to_group:
            return 'to-group'
        else:
            return 'from-group'

    @property
    def sync_device_to_group(self):
        result = self._cast_to_bool(self._values['sync_device_to_group'])
        return result

    @property
    def sync_group_to_device(self):
        result = self._cast_to_bool(self._values['sync_group_to_device'])
        return result

    @property
    def force_full_push(self):
        if self.overwrite_config:
            return 'force-full-load-push'
        else:
            return ''

    @property
    def overwrite_config(self):
        result = self._cast_to_bool(self._values['overwrite_config'])
        return result

    def _cast_to_bool(self, value):
        if value is None:
            return None
        elif value in BOOLEANS_TRUE:
            return True
        else:
            return False


class Changes(Parameters):
    def to_return(self):
        result = {}
        try:
            for returnable in self.returnables:
                result[returnable] = getattr(self, returnable)
            result = self._filter_params(result)
        except Exception:
            pass
        return result


class UsableChanges(Changes):
    pass


class ReportableChanges(Changes):
    pass


class Difference(object):
    pass


class ModuleManager(object):
    def __init__(self, *args, **kwargs):
        self.module = kwargs.get('module', None)
        self.client = F5RestClient(**self.module.params)
        self.want = ModuleParameters(params=self.module.params)
        self.changes = UsableChanges()

    def _announce_deprecations(self, result):
        warnings = result.pop('__warnings', [])
        for warning in warnings:
            self.client.module.deprecate(
                msg=warning['msg'],
                version=warning['version']
            )

    def exec_module(self):
        result = dict()

        changed = self.present()

        reportable = ReportableChanges(params=self.changes.to_return())
        changes = reportable.to_return()
        result.update(**changes)
        result.update(dict(changed=changed))
        self._announce_deprecations(result)
        return result

    def present(self):
        if not self._device_group_exists():
            raise F5ModuleError(
                "The specified 'device_group' not not exist."
            )
        if self._sync_to_group_required():
            raise F5ModuleError(
                "This device group needs an initial sync. Please use "
                "'sync_device_to_group'"
            )
        if self.exists():
            return False
        else:
            return self.execute()

    def _sync_to_group_required(self):
        status = self._get_status_from_resource()
        if status == 'Awaiting Initial Sync' and self.want.sync_group_to_device:
            return True
        return False

    def _device_group_exists(self):
        uri = "https://{0}:{1}/mgmt/tm/cm/device-group/{2}".format(
            self.client.provider['server'],
            self.client.provider['server_port'],
            self.want.device_group
        )
        resp = self.client.api.get(uri)
        try:
            response = resp.json()
        except ValueError:
            return False
        if resp.status == 404 or 'code' in response and response['code'] == 404:
            return False
        return True

    def execute(self):
        self.execute_on_device()
        self._wait_for_sync()
        return True

    def exists(self):
        status = self._get_status_from_resource()
        if status == 'In Sync':
            return True
        else:
            return False

    def execute_on_device(self):
        sync_cmd = 'config-sync {0} {1} {2}'.format(
            self.want.direction,
            self.want.device_group,
            self.want.force_full_push
        )
        uri = "https://{0}:{1}/mgmt/tm/cm".format(
            self.client.provider['server'],
            self.client.provider['server_port'],
        )
        args = dict(
            command='run',
            utilCmdArgs=sync_cmd
        )
        resp = self.client.api.post(uri, json=args)

        try:
            response = resp.json()
        except ValueError as ex:
            raise F5ModuleError(str(ex))

        if 'code' in response and response['code'] == 400:
            if 'message' in response:
                raise F5ModuleError(response['message'])
            else:
                raise F5ModuleError(resp.content)

    def _wait_for_sync(self):
        # Wait no more than half an hour
        for x in range(1, 180):
            time.sleep(3)
            status = self._get_status_from_resource()

            # Changes Pending:
            #     The existing device has changes made to it that
            #     need to be sync'd to the group.
            #
            # Awaiting Initial Sync:
            #     This is a new device group and has not had any sync
            #     done yet. You _must_ `sync_device_to_group` in this
            #     case.
            #
            # Not All Devices Synced:
            #     A device group will go into this state immediately
            #     after starting the sync and stay until all devices finish.
            #
            if status in ['Changes Pending']:
                details = self._get_details_from_resource()
                self._validate_pending_status(details)
            elif status in ['Awaiting Initial Sync', 'Not All Devices Synced']:
                pass
            elif status == 'In Sync':
                return
            elif status == 'Disconnected':
                raise F5ModuleError(
                    "One or more devices are unreachable (disconnected). "
                    "Resolve any communication problems before attempting to sync."
                )
            else:
                raise F5ModuleError(status)

    def read_current_from_device(self):
        uri = "https://{0}:{1}/mgmt/tm/cm/sync-status/".format(
            self.client.provider['server'],
            self.client.provider['server_port'],
        )
        resp = self.client.api.get(uri)
        try:
            response = resp.json()
        except ValueError as ex:
            raise F5ModuleError(str(ex))

        if 'code' in response and response['code'] == 400:
            if 'message' in response:
                raise F5ModuleError(response['message'])
            else:
                raise F5ModuleError(resp.content)
        return response

    def _get_status_from_resource(self):
        resource = self.read_current_from_device()
        entries = resource['entries'].copy()
        k, v = entries.popitem()
        status = v['nestedStats']['entries']['status']['description']
        return status

    def _get_details_from_resource(self):
        resource = self.read_current_from_device()
        stats = resource['entries'].copy()
        tree = Tree(stats)
        details = list(tree.execute('$..*["details"]["description"]'))
        result = details[::-1]
        return result

    def _validate_pending_status(self, details):
        """Validate the content of a pending sync operation

        This is a hack. The REST API is not consistent with its 'status' values
        so this method is here to check the returned strings from the operation
        and see if it reported any of these inconsistencies.

        :param details:
        :raises F5ModuleError:
        """
        pattern1 = r'.*(?P<msg>Recommended\s+action.*)'
        for detail in details:
            matches = re.search(pattern1, detail)
            if matches:
                raise F5ModuleError(matches.group('msg'))


class ArgumentSpec(object):
    def __init__(self):
        self.supports_check_mode = False

        argument_spec = dict(
            sync_device_to_group=dict(
                type='bool'
            ),
            sync_most_recent_to_device=dict(
                type='bool'
            ),
            overwrite_config=dict(
                type='bool',
                default='no'
            ),
            device_group=dict(
                required=True
            )
        )
        self.argument_spec = {}
        self.argument_spec.update(f5_argument_spec)
        self.argument_spec.update(argument_spec)

        self.required_one_of = [
            ['sync_device_to_group', 'sync_most_recent_to_device']
        ]
        self.mutually_exclusive = [
            ['sync_device_to_group', 'sync_most_recent_to_device']
        ]


def main():
    spec = ArgumentSpec()

    module = AnsibleModule(
        argument_spec=spec.argument_spec,
        supports_check_mode=spec.supports_check_mode,
        mutually_exclusive=spec.mutually_exclusive,
        required_one_of=spec.required_one_of
    )

    try:
        mm = ModuleManager(module=module)
        results = mm.exec_module()
        module.exit_json(**results)
    except F5ModuleError as ex:
        module.fail_json(msg=str(ex))


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright: (c) 2018, F5 Networks Inc.
# GNU General Public License v3.0 (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'certified'}

DOCUMENTATION = r'''
---
module: bigip_asm_policy_signature_set
short_description: Manages Signature Sets on ASM policy
description:
  - Manages Signature Sets on ASM policy.
version_added: 2.8
options:
  name:
    description:
      - Specifies the name of the signature sets to apply on or remove from the ASM policy.
      - Apart from built-in signature sets that ship with the device, users can use user created
        signature sets.
      - When C(All Response Signatures), configures all signatures in the attack signature
        pool that can review responses.
      - When C(All Signatures), configures all attack signatures in the attack signature pool.
      - When C(Apache Struts Signatures), configures signatures that target attacks against
        the Apache Struts web servers. Only available in version 13.x and up.
      - When C(Apache Tomcat Signatures), configures signatures that target attacks against
        the Apache Tomcat web servers. Only available in version 13.x and up.
      - When C(Cisco Signatures), configures signatures that target attacks against Cisco systems.
        Only available in version 13.x and up.
      - When C(Command Execution Signatures), configures signatures involving attacks perpetrated by executing commands.
      - When C(Cross Site Scripting Signatures), configures signatures that target attacks caused
        by cross-site scripting techniques.
      - When C(Directory Indexing Signatures), configures signatures targeting attacks that browse directory listings.
      - When C(Generic Detection Signatures), configures signatures targeting well-known
        or common web and application attacks.
      - When C(HTTP Response Splitting Signatures), configures signatures targeting attacks that
        take advantage of responses for which input values have not been sanitized.
      - When C(High Accuracy Detection Evasion Signatures), configures signatures with a high level of accuracy
        that produce few false positives when identifying evasion attacks. Only available in version 13.x and up.
      - When C(High Accuracy Signatures), configures signatures with a high level of accuracy
        that produce few false positives when identifying evasion attacks.
      - When C(IIS and Windows Signatures), configures signatures that target attacks against IIS
        and Windows based systems. Only available in version 13.x and up.
      - When C(Information Leakage Signatures), configures signatures targeting attacks that are looking for system data
        or debugging information that shows where the system is vulnerable to attack.
      - When C(Java Servlets/JSP Signatures), configures signatures that target attacks against Java Servlets
        and Java Server Pages (JSP) based applications. Only available in version 13.x and up.
      - When C(Low Accuracy Signatures), configures signatures that may result in more false positives
        when identifying attacks.
      - When C(Medium Accuracy Signatures), configures signatures with a medium level of accuracy
        when identifying attacks.
      - When C(OS Command Injection Signatures), configures signatures targeting attacks
        that attempt to run system level commands through a vulnerable application.
      - When C(OWA Signatures), configures signatures that target attacks against
        the Microsoft Outlook Web Access (OWA) application.
      - When C(Other Application Attacks Signatures), configures signatures targeting miscellaneous attacks,
        including session fixation, local file access, injection attempts, header tampering
        and so on, affecting many applications.
      - When C(Path Traversal Signatures), configures signatures targeting attacks that attempt to access files
        and directories that are stored outside the web root folder.
      - When C(Predictable Resource Location Signatures), configures signatures targeting attacks that attempt
        to uncover hidden website content and functionality by forceful browsing, or by directory and file enumeration.
      - When C(Remote File Include Signatures), configures signatures targeting attacks that attempt to exploit
        a remote file include vulnerability that could enable a remote attacker to execute arbitrary commands
        on the server hosting the application.
      - When C(SQL Injection Signatures), configures signatures targeting attacks that attempt to insert (inject)
        a SQL query using the input data from a client to an application.
      - When C(Server Side Code Injection Signatures), configures signatures targeting code injection attacks
        on the server side.
      - When C(WebSphere signatures), configures signatures targeting attacks on many computing platforms
        that are integrated using WebSphere including general database, Microsoft Windows, IIS,
        Microsoft SQL Server, Apache, Oracle, Unix/Linux, IBM DB2, PostgreSQL, and XML.
      - When C(XPath Injection Signatures), configures signatures targeting attacks that attempt to gain access
        to data structures or bypass permissions when a web site uses user-supplied information
        to construct XPath queries for XML data.
    type: str
    required: True
  policy_name:
    description:
      - Specifies the name of an existing ASM policy to add or remove signature sets.
    type: str
    required: True
  alarm:
    description:
      - Specifies if the security policy logs the request data in the Statistics screen,
        when a request matches a signature that is included in the signature set.
    type: bool
  block:
    description:
      - Effective when the security policy`s enforcement mode is Blocking.
      - Determines how the system treats requests that match a signature included in the signature set.
      - When C(yes) the system blocks all requests that match a signature,
        and provides the client with a support ID number.
      - When C(no) the system accepts those requests.
    type: bool
  learn:
    description:
      - Specifies if the security policy learns all requests that match a signature
        that is included in the signature set.
    type: bool
  state:
    description:
      - When C(present), ensures that the resource exists.
      - When C(absent), ensures the resource is removed.
    type: str
    default: present
    choices:
      - present
      - absent
  partition:
    description:
      - This parameter is only used when identifying ASM policy.
    type: str
    default: Common
notes:
  - This module is primarily used as a component of configuring ASM policy in Ansible Galaxy ASM Policy Role.
extends_documentation_fragment: f5
author:
  - Wojciech Wypior (@wojtek0806)
'''

EXAMPLES = r'''
- name: Add Signature Set to ASM Policy
  bigip_asm_policy_signature_set:
    name: IIS and Windows Signatures
    policy_name: FooPolicy
    provider:
      password: secret
      server: lb.mydomain.com
      user: admin
  delegate_to: localhost
- name: Remove Signature Set to ASM Policy
  bigip_asm_policy_signature_set:
    name: IIS and Windows Signatures
    policy_name: FooPolicy
    state: absent
    provider:
      password: secret
      server: lb.mydomain.com
      user: admin
  delegate_to: localhost
'''

RETURN = r'''
policy_name:
  description: The name of the ASM policy
  returned: changed
  type: str
  sample: FooPolicy
name:
  description: The name of Signature Set added/removed on ASM policy
  returned: changed
  type: str
  sample: Cisco Signatures
alarm:
  description: Specifies whether the security policy logs the request data in the Statistics screen
  returned: changed
  type: bool
  sample: yes
block:
  description: Determines how the system treats requests that match a signature included in the signature set
  returned: changed
  type: bool
  sample: no
learn:
  description: Specifies if the policy learns all requests that match a signature that is included in the signature set
  returned: changed
  type: bool
  sample: yes
'''

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.basic import env_fallback
from distutils.version import LooseVersion

try:
    from library.module_utils.network.f5.bigip import F5RestClient
    from library.module_utils.network.f5.common import F5ModuleError
    from library.module_utils.network.f5.common import AnsibleF5Parameters
    from library.module_utils.network.f5.common import fq_name
    from library.module_utils.network.f5.common import transform_name
    from library.module_utils.network.f5.common import f5_argument_spec
    from library.module_utils.network.f5.common import flatten_boolean
    from library.module_utils.network.f5.icontrol import tmos_version
    from library.module_utils.network.f5.icontrol import module_provisioned
except ImportError:
    from ansible.module_utils.network.f5.bigip import F5RestClient
    from ansible.module_utils.network.f5.common import F5ModuleError
    from ansible.module_utils.network.f5.common import AnsibleF5Parameters
    from ansible.module_utils.network.f5.common import fq_name
    from ansible.module_utils.network.f5.common import transform_name
    from ansible.module_utils.network.f5.common import f5_argument_spec
    from ansible.module_utils.network.f5.common import flatten_boolean
    from ansible.module_utils.network.f5.icontrol import tmos_version
    from ansible.module_utils.network.f5.icontrol import module_provisioned


class Parameters(AnsibleF5Parameters):
    api_map = {

    }

    api_attributes = [
        'alarm',
        'block',
        'learn',

    ]

    returnables = [
        'policy_name',
        'name',
        'alarm',
        'block',
        'learn',

    ]

    updatables = [
        'alarm',
        'block',
        'learn',
    ]


class ApiParameters(Parameters):
    pass


class ModuleParameters(Parameters):
    @property
    def alarm(self):
        result = flatten_boolean(self._values['alarm'])
        if result:
            if result == 'yes':
                return True
            return False

    @property
    def block(self):
        result = flatten_boolean(self._values['block'])
        if result:
            if result == 'yes':
                return True
            return False

    @property
    def learn(self):
        result = flatten_boolean(self._values['learn'])
        if result:
            if result == 'yes':
                return True
            return False

    def _signature_set_exists_on_device(self, name):
        uri = "https://{0}:{1}/mgmt/tm/asm/signature-sets".format(
            self.client.provider['server'],
            self.client.provider['server_port'],
        )

        query = "?$select=name"
        resp = self.client.api.get(uri + query)

        try:
            response = resp.json()
        except ValueError as ex:
            raise F5ModuleError(str(ex))

        if 'code' in response and response['code'] == 400:
            if 'message' in response:
                raise F5ModuleError(response['message'])
            else:
                raise F5ModuleError(resp.content)

        if any(p['name'] == name for p in response['items']):
            return True
        return False

    @property
    def name(self):
        if self._values['name'] is None:
            return None

        version = tmos_version(self.client)

        if LooseVersion(version) < LooseVersion('13.0.0'):
            name_list = [
                'All Response Signatures',
                'All Signatures',
                'Command Execution Signatures',
                'Cross Site Scripting Signatures',
                'Directory Indexing Signatures',
                'Generic Detection Signatures',
                'HTTP Response Splitting Signatures',
                'High Accuracy Signatures',
                'Information Leakage Signatures',
                'Low Accuracy Signatures',
                'Medium Accuracy Signatures',
                'OS Command Injection Signatures',
                'OWA Signatures',
                'Other Application Attacks Signatures',
                'Path Traversal Signatures',
                'Predictable Resource Location Signatures',
                'Remote File Include Signatures',
                'SQL Injection Signatures',
                'Server Side Code Injection Signatures',
                'WebSphere signatures',
                'XPath Injection Signatures'
            ]
        else:
            name_list = [
                'All Response Signatures',
                'All Signatures',
                'Apache Struts Signatures',
                'Apache Tomcat Signatures',
                'Cisco Signatures',
                'Command Execution Signatures',
                'Cross Site Scripting Signatures',
                'Directory Indexing Signatures',
                'Generic Detection Signatures',
                'HTTP Response Splitting Signatures',
                'High Accuracy Detection Evasion Signatures',
                'High Accuracy Signatures',
                'IIS and Windows Signatures',
                'Information Leakage Signatures',
                'Java Servlets/JSP Signatures',
                'Low Accuracy Signatures',
                'Medium Accuracy Signatures',
                'OS Command Injection Signatures',
                'OWA Signatures',
                'Other Application Attacks Signatures',
                'Path Traversal Signatures',
                'Predictable Resource Location Signatures',
                'Remote File Include Signatures',
                'SQL Injection Signatures',
                'Server Side Code Injection Signatures',
                'WebSphere signatures',
                'XPath Injection Signatures'
            ]

        if self._values['name'] in name_list:
            return self._values['name']

        if self._signature_set_exists_on_device(self._values['name']):
            return self._values['name']

        raise F5ModuleError(
            "The specified signature {0} set does not exist.".format(
                self._values['name']
            )
        )


class Changes(Parameters):
    def to_return(self):
        result = {}
        try:
            for returnable in self.returnables:
                result[returnable] = getattr(self, returnable)
            result = self._filter_params(result)
        except Exception:
            pass
        return result


class UsableChanges(Changes):
    pass


class ReportableChanges(Changes):
    @property
    def alarm(self):
        return flatten_boolean(self._values['alarm'])

    @property
    def learn(self):
        return flatten_boolean(self._values['learn'])

    @property
    def block(self):
        return flatten_boolean(self._values['block'])


class Difference(object):
    def __init__(self, want, have=None):
        self.want = want
        self.have = have

    def compare(self, param):
        try:
            result = getattr(self, param)
            return result
        except AttributeError:
            return self.__default(param)

    def __default(self, param):
        attr1 = getattr(self.want, param)
        try:
            attr2 = getattr(self.have, param)
            if attr1 != attr2:
                return attr1
        except AttributeError:
            return attr1


class ModuleManager(object):
    def __init__(self, *args, **kwargs):
        self.module = kwargs.get('module', None)
        self.client = F5RestClient(**self.module.params)
        self.want = ModuleParameters(params=self.module.params, client=self.client)
        self.have = ApiParameters()
        self.changes = UsableChanges()

    def _set_changed_options(self):
        changed = {}
        for key in Parameters.returnables:
            if getattr(self.want, key) is not None:
                changed[key] = getattr(self.want, key)
        if changed:
            self.changes = UsableChanges(params=changed)

    def _update_changed_options(self):
        diff = Difference(self.want, self.have)
        updatables = Parameters.updatables
        changed = dict()
        for k in updatables:
            change = diff.compare(k)
            if change is None:
                continue
            else:
                if isinstance(change, dict):
                    changed.update(change)
                else:
                    changed[k] = change
        if changed:
            self.changes = UsableChanges(params=changed)
            return True
        return False

    def _announce_deprecations(self, result):
        warnings = result.pop('__warnings', [])
        for warning in warnings:
            self.client.module.deprecate(
                msg=warning['msg'],
                version=warning['version']
            )

    def exec_module(self):
        if not module_provisioned(self.client, 'asm'):
            raise F5ModuleError(
                "ASM must be provisioned to use this module."
            )
        changed = False
        result = dict()
        state = self.want.state

        if state == "present":
            changed = self.present()
        elif state == "absent":
            changed = self.absent()

        reportable = ReportableChanges(params=self.changes.to_return())
        changes = reportable.to_return()
        result.update(**changes)
        result.update(dict(changed=changed))
        self._announce_deprecations(result)
        return result

    def present(self):
        if self.exists():
            return self.update()
        else:
            return self.create()

    def absent(self):
        if self.exists():
            return self.remove()
        return False

    def should_update(self):
        result = self._update_changed_options()
        if result:
            return True
        return False

    def update(self):
        self.have = self.read_current_from_device()
        if not self.should_update():
            return False
        if self.module.check_mode:
            return True
        self.update_on_device()
        return True

    def remove(self):
        if self.module.check_mode:
            return True
        self.remove_from_device()
        if self.exists():
            raise F5ModuleError("Failed to delete the resource.")
        return True

    def create(self):
        self._set_changed_options()
        if self.module.check_mode:
            return True
        self.create_on_device()
        return True

    def exists(self):
        policy_id = self._get_policy_id()
        set_link = self._get_signature_set_link()
        uri = 'https://{0}:{1}/mgmt/tm/asm/policies/{2}/signature-sets/'.format(
            self.client.provider['server'],
            self.client.provider['server_port'],
            policy_id,
        )
        resp = self.client.api.get(uri)

        try:
            response = resp.json()
        except ValueError as ex:
            raise F5ModuleError(str(ex))

        if 'items' in response and response['items'] != []:
            for st in response['items']:
                if st['signatureSetReference'] == set_link:
                    self.want.ss_id = st['id']
                    return True
        return False

    def _get_signature_set_link(self):
        result = None
        signature_set = self.want.name
        uri = "https://{0}:{1}/mgmt/tm/asm/signature-sets".format(
            self.client.provider['server'],
            self.client.provider['server_port'],
        )

        query = "?$select=name"
        resp = self.client.api.get(uri + query)

        try:
            response = resp.json()
        except ValueError as ex:
            raise F5ModuleError(str(ex))

        if 'code' in response and response['code'] == 400:
            if 'message' in response:
                raise F5ModuleError(response['message'])
            else:
                raise F5ModuleError(resp.content)

        if 'items' in response and response['items'] != []:
            for item in response['items']:
                if item['name'] == signature_set:
                    result = dict(link=item['selfLink'])

        return result

    def _get_policy_id(self):
        policy_id = None
        uri = "https://{0}:{1}/mgmt/tm/asm/policies/".format(
            self.client.provider['server'],
            self.client.provider['server_port'],
        )
        query = "?$filter=contains(name,'{0}')+and+contains(partition,'{1}')&$select=name,id".format(
            self.want.policy_name, self.want.partition
        )
        resp = self.client.api.get(uri + query)

        try:
            response = resp.json()
        except ValueError as ex:
            raise F5ModuleError(str(ex))

        if 'items' in response and response['items'] != []:
            policy_id = response['items'][0]['id']

        if not policy_id:
            raise F5ModuleError(
                "The policy with the name {0} does not exist".format(self.want.policy_name)
            )
        return policy_id

    def create_on_device(self):
        policy_id = self._get_policy_id()
        params = self.changes.api_params()
        params['signatureSetReference'] = self._get_signature_set_link()
        uri = "https://{0}:{1}/mgmt/tm/asm/policies/{2}/signature-sets/".format(
            self.client.provider['server'],
            self.client.provider['server_port'],
            policy_id
        )
        resp = self.client.api.post(uri, json=params)
        try:
            response = resp.json()
        except ValueError as ex:
            raise F5ModuleError(str(ex))

        if 'code' in response and response['code'] in [400, 409]:
            if 'message' in response:
                raise F5ModuleError(response['message'])
            else:
                raise F5ModuleError(resp.content)
        return True

    def update_on_device(self):
        policy_id = self._get_policy_id()
        params = self.changes.api_params()
        uri = "https://{0}:{1}/mgmt/tm/asm/policies/{2}/signature-sets/{3}".format(
            self.client.provider['server'],
            self.client.provider['server_port'],
            policy_id,
            self.want.ss_id
        )
        resp = self.client.api.patch(uri, json=params)
        try:
            response = resp.json()
        except ValueError as ex:
            raise F5ModuleError(str(ex))

        if 'code' in response and response['code'] == 400:
            if 'message' in response:
                raise F5ModuleError(response['message'])
            else:
                raise F5ModuleError(resp.content)

    def remove_from_device(self):
        policy_id = self._get_policy_id()
        uri = 'https://{0}:{1}/mgmt/tm/asm/policies/{2}/signature-sets/{3}'.format(
            self.client.provider['server'],
            self.client.provider['server_port'],
            policy_id,
            self.want.ss_id
        )
        response = self.client.api.delete(uri)
        if response.status in [200, 201]:
            return True
        raise F5ModuleError(response.content)

    def read_current_from_device(self):
        policy_id = self._get_policy_id()
        uri = "https://{0}:{1}/mgmt/tm/asm/policies/{2}/signature-sets/{3}".format(
            self.client.provider['server'],
            self.client.provider['server_port'],
            policy_id,
            self.want.ss_id
        )
        resp = self.client.api.get(uri)
        try:
            response = resp.json()
        except ValueError as ex:
            raise F5ModuleError(str(ex))

        if 'code' in response and response['code'] == 400:
            if 'message' in response:
                raise F5ModuleError(response['message'])
            else:
                raise F5ModuleError(resp.content)
        return ApiParameters(params=response)


class ArgumentSpec(object):
    def __init__(self):
        self.supports_check_mode = True
        argument_spec = dict(
            policy_name=dict(
                required=True
            ),
            name=dict(
                required=True
            ),
            alarm=dict(
                type='bool'
            ),
            block=dict(
                type='bool'
            ),
            learn=dict(
                type='bool'
            ),
            state=dict(
                default='present',
                choices=['present', 'absent']
            ),
            partition=dict(
                default='Common',
                fallback=(env_fallback, ['F5_PARTITION'])
            )
        )
        self.argument_spec = {}
        self.argument_spec.update(f5_argument_spec)
        self.argument_spec.update(argument_spec)


def main():
    spec = ArgumentSpec()

    module = AnsibleModule(
        argument_spec=spec.argument_spec,
        supports_check_mode=spec.supports_check_mode,
    )

    try:
        mm = ModuleManager(module=module)
        results = mm.exec_module()
        module.exit_json(**results)
    except F5ModuleError as ex:
        module.fail_json(msg=str(ex))


if __name__ == '__main__':
    main()



#!/usr/bin/python
from __future__ import (absolute_import, division, print_function)
# Copyright 2019 Fortinet, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# the lib use python logging can get it if the following is set in your
# Ansible config.

__metaclass__ = type

ANSIBLE_METADATA = {'status': ['preview'],
                    'supported_by': 'community',
                    'metadata_version': '1.1'}

DOCUMENTATION = '''
---
module: fortios_firewall_interface_policy6
short_description: Configure IPv6 interface policies in Fortinet's FortiOS and FortiGate.
description:
    - This module is able to configure a FortiGate or FortiOS by
      allowing the user to configure firewall feature and interface_policy6 category.
      Examples includes all options and need to be adjusted to datasources before usage.
      Tested with FOS v6.0.2
version_added: "2.8"
author:
    - Miguel Angel Munoz (@mamunozgonzalez)
    - Nicolas Thomas (@thomnico)
notes:
    - Requires fortiosapi library developed by Fortinet
    - Run as a local_action in your playbook
requirements:
    - fortiosapi>=0.9.8
options:
    host:
       description:
            - FortiOS or FortiGate ip address.
       required: true
    username:
        description:
            - FortiOS or FortiGate username.
        required: true
    password:
        description:
            - FortiOS or FortiGate password.
        default: ""
    vdom:
        description:
            - Virtual domain, among those defined previously. A vdom is a
              virtual instance of the FortiGate that can be configured and
              used as a different unit.
        default: root
    https:
        description:
            - Indicates if the requests towards FortiGate must use HTTPS
              protocol
        type: bool
        default: false
    firewall_interface_policy6:
        description:
            - Configure IPv6 interface policies.
        default: null
        suboptions:
            state:
                description:
                    - Indicates whether to create or remove the object
                choices:
                    - present
                    - absent
            address-type:
                description:
                    - Policy address type (IPv4 or IPv6).
                choices:
                    - ipv4
                    - ipv6
            application-list:
                description:
                    - Application list name. Source application.list.name.
            application-list-status:
                description:
                    - Enable/disable application control.
                choices:
                    - enable
                    - disable
            av-profile:
                description:
                    - Antivirus profile. Source antivirus.profile.name.
            av-profile-status:
                description:
                    - Enable/disable antivirus.
                choices:
                    - enable
                    - disable
            comments:
                description:
                    - Comments.
            dlp-sensor:
                description:
                    - DLP sensor name. Source dlp.sensor.name.
            dlp-sensor-status:
                description:
                    - Enable/disable DLP.
                choices:
                    - enable
                    - disable
            dsri:
                description:
                    - Enable/disable DSRI.
                choices:
                    - enable
                    - disable
            dstaddr6:
                description:
                    - IPv6 address object to limit traffic monitoring to network traffic sent to the specified address or range.
                suboptions:
                    name:
                        description:
                            - Address name. Source firewall.address6.name firewall.addrgrp6.name.
                        required: true
            interface:
                description:
                    - Monitored interface name from available interfaces. Source system.zone.name system.interface.name.
            ips-sensor:
                description:
                    - IPS sensor name. Source ips.sensor.name.
            ips-sensor-status:
                description:
                    - Enable/disable IPS.
                choices:
                    - enable
                    - disable
            label:
                description:
                    - Label.
            logtraffic:
                description:
                    - "Logging type to be used in this policy (Options: all | utm | disable, Default: utm)."
                choices:
                    - all
                    - utm
                    - disable
            policyid:
                description:
                    - Policy ID.
                required: true
            scan-botnet-connections:
                description:
                    - Enable/disable scanning for connections to Botnet servers.
                choices:
                    - disable
                    - block
                    - monitor
            service6:
                description:
                    - Service name.
                suboptions:
                    name:
                        description:
                            - Address name. Source firewall.service.custom.name firewall.service.group.name.
                        required: true
            spamfilter-profile:
                description:
                    - Antispam profile. Source spamfilter.profile.name.
            spamfilter-profile-status:
                description:
                    - Enable/disable antispam.
                choices:
                    - enable
                    - disable
            srcaddr6:
                description:
                    - IPv6 address object to limit traffic monitoring to network traffic sent from the specified address or range.
                suboptions:
                    name:
                        description:
                            - Address name. Source firewall.address6.name firewall.addrgrp6.name.
                        required: true
            status:
                description:
                    - Enable/disable this policy.
                choices:
                    - enable
                    - disable
            webfilter-profile:
                description:
                    - Web filter profile. Source webfilter.profile.name.
            webfilter-profile-status:
                description:
                    - Enable/disable web filtering.
                choices:
                    - enable
                    - disable
'''

EXAMPLES = '''
- hosts: localhost
  vars:
   host: "192.168.122.40"
   username: "admin"
   password: ""
   vdom: "root"
  tasks:
  - name: Configure IPv6 interface policies.
    fortios_firewall_interface_policy6:
      host:  "{{ host }}"
      username: "{{ username }}"
      password: "{{ password }}"
      vdom:  "{{ vdom }}"
      firewall_interface_policy6:
        state: "present"
        address-type: "ipv4"
        application-list: "<your_own_value> (source application.list.name)"
        application-list-status: "enable"
        av-profile: "<your_own_value> (source antivirus.profile.name)"
        av-profile-status: "enable"
        comments: "<your_own_value>"
        dlp-sensor: "<your_own_value> (source dlp.sensor.name)"
        dlp-sensor-status: "enable"
        dsri: "enable"
        dstaddr6:
         -
            name: "default_name_13 (source firewall.address6.name firewall.addrgrp6.name)"
        interface: "<your_own_value> (source system.zone.name system.interface.name)"
        ips-sensor: "<your_own_value> (source ips.sensor.name)"
        ips-sensor-status: "enable"
        label: "<your_own_value>"
        logtraffic: "all"
        policyid: "19"
        scan-botnet-connections: "disable"
        service6:
         -
            name: "default_name_22 (source firewall.service.custom.name firewall.service.group.name)"
        spamfilter-profile: "<your_own_value> (source spamfilter.profile.name)"
        spamfilter-profile-status: "enable"
        srcaddr6:
         -
            name: "default_name_26 (source firewall.address6.name firewall.addrgrp6.name)"
        status: "enable"
        webfilter-profile: "<your_own_value> (source webfilter.profile.name)"
        webfilter-profile-status: "enable"
'''

RETURN = '''
build:
  description: Build number of the fortigate image
  returned: always
  type: str
  sample: '1547'
http_method:
  description: Last method used to provision the content into FortiGate
  returned: always
  type: str
  sample: 'PUT'
http_status:
  description: Last result given by FortiGate on last operation applied
  returned: always
  type: str
  sample: "200"
mkey:
  description: Master key (id) used in the last call to FortiGate
  returned: success
  type: str
  sample: "id"
name:
  description: Name of the table used to fulfill the request
  returned: always
  type: str
  sample: "urlfilter"
path:
  description: Path of the table used to fulfill the request
  returned: always
  type: str
  sample: "webfilter"
revision:
  description: Internal revision number
  returned: always
  type: str
  sample: "17.0.2.10658"
serial:
  description: Serial number of the unit
  returned: always
  type: str
  sample: "FGVMEVYYQT3AB5352"
status:
  description: Indication of the operation's result
  returned: always
  type: str
  sample: "success"
vdom:
  description: Virtual domain used
  returned: always
  type: str
  sample: "root"
version:
  description: Version of the FortiGate
  returned: always
  type: str
  sample: "v5.6.3"

'''

from ansible.module_utils.basic import AnsibleModule

fos = None


def login(data):
    host = data['host']
    username = data['username']
    password = data['password']

    fos.debug('on')
    if 'https' in data and not data['https']:
        fos.https('off')
    else:
        fos.https('on')

    fos.login(host, username, password)


def filter_firewall_interface_policy6_data(json):
    option_list = ['address-type', 'application-list', 'application-list-status',
                   'av-profile', 'av-profile-status', 'comments',
                   'dlp-sensor', 'dlp-sensor-status', 'dsri',
                   'dstaddr6', 'interface', 'ips-sensor',
                   'ips-sensor-status', 'label', 'logtraffic',
                   'policyid', 'scan-botnet-connections', 'service6',
                   'spamfilter-profile', 'spamfilter-profile-status', 'srcaddr6',
                   'status', 'webfilter-profile', 'webfilter-profile-status']
    dictionary = {}

    for attribute in option_list:
        if attribute in json and json[attribute] is not None:
            dictionary[attribute] = json[attribute]

    return dictionary


def firewall_interface_policy6(data, fos):
    vdom = data['vdom']
    firewall_interface_policy6_data = data['firewall_interface_policy6']
    filtered_data = filter_firewall_interface_policy6_data(firewall_interface_policy6_data)
    if firewall_interface_policy6_data['state'] == "present":
        return fos.set('firewall',
                       'interface-policy6',
                       data=filtered_data,
                       vdom=vdom)

    elif firewall_interface_policy6_data['state'] == "absent":
        return fos.delete('firewall',
                          'interface-policy6',
                          mkey=filtered_data['policyid'],
                          vdom=vdom)


def fortios_firewall(data, fos):
    login(data)

    methodlist = ['firewall_interface_policy6']
    for method in methodlist:
        if data[method]:
            resp = eval(method)(data, fos)
            break

    fos.logout()
    return not resp['status'] == "success", resp['status'] == "success", resp


def main():
    fields = {
        "host": {"required": True, "type": "str"},
        "username": {"required": True, "type": "str"},
        "password": {"required": False, "type": "str", "no_log": True},
        "vdom": {"required": False, "type": "str", "default": "root"},
        "https": {"required": False, "type": "bool", "default": "False"},
        "firewall_interface_policy6": {
            "required": False, "type": "dict",
            "options": {
                "state": {"required": True, "type": "str",
                          "choices": ["present", "absent"]},
                "address-type": {"required": False, "type": "str",
                                 "choices": ["ipv4", "ipv6"]},
                "application-list": {"required": False, "type": "str"},
                "application-list-status": {"required": False, "type": "str",
                                            "choices": ["enable", "disable"]},
                "av-profile": {"required": False, "type": "str"},
                "av-profile-status": {"required": False, "type": "str",
                                      "choices": ["enable", "disable"]},
                "comments": {"required": False, "type": "str"},
                "dlp-sensor": {"required": False, "type": "str"},
                "dlp-sensor-status": {"required": False, "type": "str",
                                      "choices": ["enable", "disable"]},
                "dsri": {"required": False, "type": "str",
                         "choices": ["enable", "disable"]},
                "dstaddr6": {"required": False, "type": "list",
                             "options": {
                                 "name": {"required": True, "type": "str"}
                             }},
                "interface": {"required": False, "type": "str"},
                "ips-sensor": {"required": False, "type": "str"},
                "ips-sensor-status": {"required": False, "type": "str",
                                      "choices": ["enable", "disable"]},
                "label": {"required": False, "type": "str"},
                "logtraffic": {"required": False, "type": "str",
                               "choices": ["all", "utm", "disable"]},
                "policyid": {"required": True, "type": "int"},
                "scan-botnet-connections": {"required": False, "type": "str",
                                            "choices": ["disable", "block", "monitor"]},
                "service6": {"required": False, "type": "list",
                             "options": {
                                 "name": {"required": True, "type": "str"}
                             }},
                "spamfilter-profile": {"required": False, "type": "str"},
                "spamfilter-profile-status": {"required": False, "type": "str",
                                              "choices": ["enable", "disable"]},
                "srcaddr6": {"required": False, "type": "list",
                             "options": {
                                 "name": {"required": True, "type": "str"}
                             }},
                "status": {"required": False, "type": "str",
                           "choices": ["enable", "disable"]},
                "webfilter-profile": {"required": False, "type": "str"},
                "webfilter-profile-status": {"required": False, "type": "str",
                                             "choices": ["enable", "disable"]}

            }
        }
    }

    module = AnsibleModule(argument_spec=fields,
                           supports_check_mode=False)
    try:
        from fortiosapi import FortiOSAPI
    except ImportError:
        module.fail_json(msg="fortiosapi module is required")

    global fos
    fos = FortiOSAPI()

    is_error, has_changed, result = fortios_firewall(module.params, fos)

    if not is_error:
        module.exit_json(changed=has_changed, meta=result)
    else:
        module.fail_json(msg="Error in repo", meta=result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
from __future__ import (absolute_import, division, print_function)
# Copyright 2019 Fortinet, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# the lib use python logging can get it if the following is set in your
# Ansible config.

__metaclass__ = type

ANSIBLE_METADATA = {'status': ['preview'],
                    'supported_by': 'community',
                    'metadata_version': '1.1'}

DOCUMENTATION = '''
---
module: fortios_ips_global
short_description: Configure IPS global parameter in Fortinet's FortiOS and FortiGate.
description:
    - This module is able to configure a FortiGate or FortiOS by
      allowing the user to configure ips feature and global category.
      Examples includes all options and need to be adjusted to datasources before usage.
      Tested with FOS v6.0.2
version_added: "2.8"
author:
    - Miguel Angel Munoz (@mamunozgonzalez)
    - Nicolas Thomas (@thomnico)
notes:
    - Requires fortiosapi library developed by Fortinet
    - Run as a local_action in your playbook
requirements:
    - fortiosapi>=0.9.8
options:
    host:
       description:
            - FortiOS or FortiGate ip adress.
       required: true
    username:
        description:
            - FortiOS or FortiGate username.
        required: true
    password:
        description:
            - FortiOS or FortiGate password.
        default: ""
    vdom:
        description:
            - Virtual domain, among those defined previously. A vdom is a
              virtual instance of the FortiGate that can be configured and
              used as a different unit.
        default: root
    https:
        description:
            - Indicates if the requests towards FortiGate must use HTTPS
              protocol
        type: bool
        default: true
    ips_global:
        description:
            - Configure IPS global parameter.
        default: null
        suboptions:
            anomaly-mode:
                description:
                    - Global blocking mode for rate-based anomalies.
                choices:
                    - periodical
                    - continuous
            database:
                description:
                    - Regular or extended IPS database. Regular protects against the latest common and in-the-wild attacks. Extended includes protection from
                       legacy attacks.
                choices:
                    - regular
                    - extended
            deep-app-insp-db-limit:
                description:
                    - Limit on number of entries in deep application inspection database (1 - 2147483647, 0 = use recommended setting)
            deep-app-insp-timeout:
                description:
                    - Timeout for Deep application inspection (1 - 2147483647 sec., 0 = use recommended setting).
            engine-count:
                description:
                    - Number of IPS engines running. If set to the default value of 0, FortiOS sets the number to optimize performance depending on the number
                       of CPU cores.
            exclude-signatures:
                description:
                    - Excluded signatures.
                choices:
                    - none
                    - industrial
            fail-open:
                description:
                    - Enable to allow traffic if the IPS process crashes. Default is disable and IPS traffic is blocked when the IPS process crashes.
                choices:
                    - enable
                    - disable
            intelligent-mode:
                description:
                    - Enable/disable IPS adaptive scanning (intelligent mode). Intelligent mode optimizes the scanning method for the type of traffic.
                choices:
                    - enable
                    - disable
            session-limit-mode:
                description:
                    - Method of counting concurrent sessions used by session limit anomalies. Choose between greater accuracy (accurate) or improved
                       performance (heuristics).
                choices:
                    - accurate
                    - heuristic
            skype-client-public-ipaddr:
                description:
                    - Public IP addresses of your network that receive Skype sessions. Helps identify Skype sessions. Separate IP addresses with commas.
            socket-size:
                description:
                    - IPS socket buffer size (0 - 256 MB). Default depends on available memory. Can be changed to tune performance.
            sync-session-ttl:
                description:
                    - Enable/disable use of kernel session TTL for IPS sessions.
                choices:
                    - enable
                    - disable
            traffic-submit:
                description:
                    - Enable/disable submitting attack data found by this FortiGate to FortiGuard.
                choices:
                    - enable
                    - disable
'''

EXAMPLES = '''
- hosts: localhost
  vars:
   host: "192.168.122.40"
   username: "admin"
   password: ""
   vdom: "root"
  tasks:
  - name: Configure IPS global parameter.
    fortios_ips_global:
      host:  "{{ host }}"
      username: "{{ username }}"
      password: "{{ password }}"
      vdom:  "{{ vdom }}"
      https: "False"
      ips_global:
        anomaly-mode: "periodical"
        database: "regular"
        deep-app-insp-db-limit: "5"
        deep-app-insp-timeout: "6"
        engine-count: "7"
        exclude-signatures: "none"
        fail-open: "enable"
        intelligent-mode: "enable"
        session-limit-mode: "accurate"
        skype-client-public-ipaddr: "<your_own_value>"
        socket-size: "13"
        sync-session-ttl: "enable"
        traffic-submit: "enable"
'''

RETURN = '''
build:
  description: Build number of the fortigate image
  returned: always
  type: str
  sample: '1547'
http_method:
  description: Last method used to provision the content into FortiGate
  returned: always
  type: str
  sample: 'PUT'
http_status:
  description: Last result given by FortiGate on last operation applied
  returned: always
  type: str
  sample: "200"
mkey:
  description: Master key (id) used in the last call to FortiGate
  returned: success
  type: str
  sample: "id"
name:
  description: Name of the table used to fulfill the request
  returned: always
  type: str
  sample: "urlfilter"
path:
  description: Path of the table used to fulfill the request
  returned: always
  type: str
  sample: "webfilter"
revision:
  description: Internal revision number
  returned: always
  type: str
  sample: "17.0.2.10658"
serial:
  description: Serial number of the unit
  returned: always
  type: str
  sample: "FGVMEVYYQT3AB5352"
status:
  description: Indication of the operation's result
  returned: always
  type: str
  sample: "success"
vdom:
  description: Virtual domain used
  returned: always
  type: str
  sample: "root"
version:
  description: Version of the FortiGate
  returned: always
  type: str
  sample: "v5.6.3"

'''

from ansible.module_utils.basic import AnsibleModule

fos = None


def login(data):
    host = data['host']
    username = data['username']
    password = data['password']

    fos.debug('on')
    if 'https' in data and not data['https']:
        fos.https('off')
    else:
        fos.https('on')

    fos.login(host, username, password)


def filter_ips_global_data(json):
    option_list = ['anomaly-mode', 'database', 'deep-app-insp-db-limit',
                   'deep-app-insp-timeout', 'engine-count', 'exclude-signatures',
                   'fail-open', 'intelligent-mode', 'session-limit-mode',
                   'skype-client-public-ipaddr', 'socket-size', 'sync-session-ttl',
                   'traffic-submit']
    dictionary = {}

    for attribute in option_list:
        if attribute in json and json[attribute] is not None:
            dictionary[attribute] = json[attribute]

    return dictionary


def ips_global(data, fos):
    vdom = data['vdom']
    ips_global_data = data['ips_global']
    filtered_data = filter_ips_global_data(ips_global_data)
    return fos.set('ips',
                   'global',
                   data=filtered_data,
                   vdom=vdom)


def fortios_ips(data, fos):
    login(data)

    methodlist = ['ips_global']
    for method in methodlist:
        if data[method]:
            resp = eval(method)(data, fos)
            break

    fos.logout()
    return not resp['status'] == "success", resp['status'] == "success", resp


def main():
    fields = {
        "host": {"required": True, "type": "str"},
        "username": {"required": True, "type": "str"},
        "password": {"required": False, "type": "str", "no_log": True},
        "vdom": {"required": False, "type": "str", "default": "root"},
        "https": {"required": False, "type": "bool", "default": True},
        "ips_global": {
            "required": False, "type": "dict",
            "options": {
                "anomaly-mode": {"required": False, "type": "str",
                                 "choices": ["periodical", "continuous"]},
                "database": {"required": False, "type": "str",
                             "choices": ["regular", "extended"]},
                "deep-app-insp-db-limit": {"required": False, "type": "int"},
                "deep-app-insp-timeout": {"required": False, "type": "int"},
                "engine-count": {"required": False, "type": "int"},
                "exclude-signatures": {"required": False, "type": "str",
                                       "choices": ["none", "industrial"]},
                "fail-open": {"required": False, "type": "str",
                              "choices": ["enable", "disable"]},
                "intelligent-mode": {"required": False, "type": "str",
                                     "choices": ["enable", "disable"]},
                "session-limit-mode": {"required": False, "type": "str",
                                       "choices": ["accurate", "heuristic"]},
                "skype-client-public-ipaddr": {"required": False, "type": "str"},
                "socket-size": {"required": False, "type": "int"},
                "sync-session-ttl": {"required": False, "type": "str",
                                     "choices": ["enable", "disable"]},
                "traffic-submit": {"required": False, "type": "str",
                                   "choices": ["enable", "disable"]}

            }
        }
    }

    module = AnsibleModule(argument_spec=fields,
                           supports_check_mode=False)
    try:
        from fortiosapi import FortiOSAPI
    except ImportError:
        module.fail_json(msg="fortiosapi module is required")

    global fos
    fos = FortiOSAPI()

    is_error, has_changed, result = fortios_ips(module.params, fos)

    if not is_error:
        module.exit_json(changed=has_changed, meta=result)
    else:
        module.fail_json(msg="Error in repo", meta=result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
from __future__ import (absolute_import, division, print_function)
# Copyright 2019 Fortinet, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# the lib use python logging can get it if the following is set in your
# Ansible config.

__metaclass__ = type

ANSIBLE_METADATA = {'status': ['preview'],
                    'supported_by': 'community',
                    'metadata_version': '1.1'}

DOCUMENTATION = '''
---
module: fortios_vpn_ipsec_phase2_interface
short_description: Configure VPN autokey tunnel in Fortinet's FortiOS and FortiGate.
description:
    - This module is able to configure a FortiGate or FortiOS by allowing the
      user to set and modify vpn_ipsec feature and phase2_interface category.
      Examples include all parameters and values need to be adjusted to datasources before usage.
      Tested with FOS v6.0.2
version_added: "2.8"
author:
    - Miguel Angel Munoz (@mamunozgonzalez)
    - Nicolas Thomas (@thomnico)
notes:
    - Requires fortiosapi library developed by Fortinet
    - Run as a local_action in your playbook
requirements:
    - fortiosapi>=0.9.8
options:
    host:
       description:
            - FortiOS or FortiGate ip address.
       required: true
    username:
        description:
            - FortiOS or FortiGate username.
        required: true
    password:
        description:
            - FortiOS or FortiGate password.
        default: ""
    vdom:
        description:
            - Virtual domain, among those defined previously. A vdom is a
              virtual instance of the FortiGate that can be configured and
              used as a different unit.
        default: root
    https:
        description:
            - Indicates if the requests towards FortiGate must use HTTPS
              protocol
        type: bool
        default: true
    vpn_ipsec_phase2_interface:
        description:
            - Configure VPN autokey tunnel.
        default: null
        suboptions:
            state:
                description:
                    - Indicates whether to create or remove the object
                choices:
                    - present
                    - absent
            add-route:
                description:
                    - Enable/disable automatic route addition.
                choices:
                    - phase1
                    - enable
                    - disable
            auto-discovery-forwarder:
                description:
                    - Enable/disable forwarding short-cut messages.
                choices:
                    - phase1
                    - enable
                    - disable
            auto-discovery-sender:
                description:
                    - Enable/disable sending short-cut messages.
                choices:
                    - phase1
                    - enable
                    - disable
            auto-negotiate:
                description:
                    - Enable/disable IPsec SA auto-negotiation.
                choices:
                    - enable
                    - disable
            comments:
                description:
                    - Comment.
            dhcp-ipsec:
                description:
                    - Enable/disable DHCP-IPsec.
                choices:
                    - enable
                    - disable
            dhgrp:
                description:
                    - Phase2 DH group.
                choices:
                    - 1
                    - 2
                    - 5
                    - 14
                    - 15
                    - 16
                    - 17
                    - 18
                    - 19
                    - 20
                    - 21
                    - 27
                    - 28
                    - 29
                    - 30
                    - 31
            dst-addr-type:
                description:
                    - Remote proxy ID type.
                choices:
                    - subnet
                    - range
                    - ip
                    - name
                    - subnet6
                    - range6
                    - ip6
                    - name6
            dst-end-ip:
                description:
                    - Remote proxy ID IPv4 end.
            dst-end-ip6:
                description:
                    - Remote proxy ID IPv6 end.
            dst-name:
                description:
                    - Remote proxy ID name. Source firewall.address.name firewall.addrgrp.name.
            dst-name6:
                description:
                    - Remote proxy ID name. Source firewall.address6.name firewall.addrgrp6.name.
            dst-port:
                description:
                    - Quick mode destination port (1 - 65535 or 0 for all).
            dst-start-ip:
                description:
                    - Remote proxy ID IPv4 start.
            dst-start-ip6:
                description:
                    - Remote proxy ID IPv6 start.
            dst-subnet:
                description:
                    - Remote proxy ID IPv4 subnet.
            dst-subnet6:
                description:
                    - Remote proxy ID IPv6 subnet.
            encapsulation:
                description:
                    - ESP encapsulation mode.
                choices:
                    - tunnel-mode
                    - transport-mode
            keepalive:
                description:
                    - Enable/disable keep alive.
                choices:
                    - enable
                    - disable
            keylife-type:
                description:
                    - Keylife type.
                choices:
                    - seconds
                    - kbs
                    - both
            keylifekbs:
                description:
                    - Phase2 key life in number of bytes of traffic (5120 - 4294967295).
            keylifeseconds:
                description:
                    - Phase2 key life in time in seconds (120 - 172800).
            l2tp:
                description:
                    - Enable/disable L2TP over IPsec.
                choices:
                    - enable
                    - disable
            name:
                description:
                    - IPsec tunnel name.
                required: true
            pfs:
                description:
                    - Enable/disable PFS feature.
                choices:
                    - enable
                    - disable
            phase1name:
                description:
                    - Phase 1 determines the options required for phase 2. Source vpn.ipsec.phase1-interface.name.
            proposal:
                description:
                    - Phase2 proposal.
                choices:
                    - null-md5
                    - null-sha1
                    - null-sha256
                    - null-sha384
                    - null-sha512
                    - des-null
                    - des-md5
                    - des-sha1
                    - des-sha256
                    - des-sha384
                    - des-sha512
            protocol:
                description:
                    - Quick mode protocol selector (1 - 255 or 0 for all).
            replay:
                description:
                    - Enable/disable replay detection.
                choices:
                    - enable
                    - disable
            route-overlap:
                description:
                    - Action for overlapping routes.
                choices:
                    - use-old
                    - use-new
                    - allow
            single-source:
                description:
                    - Enable/disable single source IP restriction.
                choices:
                    - enable
                    - disable
            src-addr-type:
                description:
                    - Local proxy ID type.
                choices:
                    - subnet
                    - range
                    - ip
                    - name
                    - subnet6
                    - range6
                    - ip6
                    - name6
            src-end-ip:
                description:
                    - Local proxy ID end.
            src-end-ip6:
                description:
                    - Local proxy ID IPv6 end.
            src-name:
                description:
                    - Local proxy ID name. Source firewall.address.name firewall.addrgrp.name.
            src-name6:
                description:
                    - Local proxy ID name. Source firewall.address6.name firewall.addrgrp6.name.
            src-port:
                description:
                    - Quick mode source port (1 - 65535 or 0 for all).
            src-start-ip:
                description:
                    - Local proxy ID start.
            src-start-ip6:
                description:
                    - Local proxy ID IPv6 start.
            src-subnet:
                description:
                    - Local proxy ID subnet.
            src-subnet6:
                description:
                    - Local proxy ID IPv6 subnet.
'''

EXAMPLES = '''
- hosts: localhost
  vars:
   host: "192.168.122.40"
   username: "admin"
   password: ""
   vdom: "root"
  tasks:
  - name: Configure VPN autokey tunnel.
    fortios_vpn_ipsec_phase2_interface:
      host:  "{{ host }}"
      username: "{{ username }}"
      password: "{{ password }}"
      vdom:  "{{ vdom }}"
      https: "False"
      vpn_ipsec_phase2_interface:
        state: "present"
        add-route: "phase1"
        auto-discovery-forwarder: "phase1"
        auto-discovery-sender: "phase1"
        auto-negotiate: "enable"
        comments: "<your_own_value>"
        dhcp-ipsec: "enable"
        dhgrp: "1"
        dst-addr-type: "subnet"
        dst-end-ip: "<your_own_value>"
        dst-end-ip6: "<your_own_value>"
        dst-name: "<your_own_value> (source firewall.address.name firewall.addrgrp.name)"
        dst-name6: "<your_own_value> (source firewall.address6.name firewall.addrgrp6.name)"
        dst-port: "15"
        dst-start-ip: "<your_own_value>"
        dst-start-ip6: "<your_own_value>"
        dst-subnet: "<your_own_value>"
        dst-subnet6: "<your_own_value>"
        encapsulation: "tunnel-mode"
        keepalive: "enable"
        keylife-type: "seconds"
        keylifekbs: "23"
        keylifeseconds: "24"
        l2tp: "enable"
        name: "default_name_26"
        pfs: "enable"
        phase1name: "<your_own_value> (source vpn.ipsec.phase1-interface.name)"
        proposal: "null-md5"
        protocol: "30"
        replay: "enable"
        route-overlap: "use-old"
        single-source: "enable"
        src-addr-type: "subnet"
        src-end-ip: "<your_own_value>"
        src-end-ip6: "<your_own_value>"
        src-name: "<your_own_value> (source firewall.address.name firewall.addrgrp.name)"
        src-name6: "<your_own_value> (source firewall.address6.name firewall.addrgrp6.name)"
        src-port: "39"
        src-start-ip: "<your_own_value>"
        src-start-ip6: "<your_own_value>"
        src-subnet: "<your_own_value>"
        src-subnet6: "<your_own_value>"
'''

RETURN = '''
build:
  description: Build number of the fortigate image
  returned: always
  type: str
  sample: '1547'
http_method:
  description: Last method used to provision the content into FortiGate
  returned: always
  type: str
  sample: 'PUT'
http_status:
  description: Last result given by FortiGate on last operation applied
  returned: always
  type: str
  sample: "200"
mkey:
  description: Master key (id) used in the last call to FortiGate
  returned: success
  type: str
  sample: "id"
name:
  description: Name of the table used to fulfill the request
  returned: always
  type: str
  sample: "urlfilter"
path:
  description: Path of the table used to fulfill the request
  returned: always
  type: str
  sample: "webfilter"
revision:
  description: Internal revision number
  returned: always
  type: str
  sample: "17.0.2.10658"
serial:
  description: Serial number of the unit
  returned: always
  type: str
  sample: "FGVMEVYYQT3AB5352"
status:
  description: Indication of the operation's result
  returned: always
  type: str
  sample: "success"
vdom:
  description: Virtual domain used
  returned: always
  type: str
  sample: "root"
version:
  description: Version of the FortiGate
  returned: always
  type: str
  sample: "v5.6.3"

'''

from ansible.module_utils.basic import AnsibleModule

fos = None


def login(data):
    host = data['host']
    username = data['username']
    password = data['password']

    fos.debug('on')
    if 'https' in data and not data['https']:
        fos.https('off')
    else:
        fos.https('on')

    fos.login(host, username, password)


def filter_vpn_ipsec_phase2_interface_data(json):
    option_list = ['add-route', 'auto-discovery-forwarder', 'auto-discovery-sender',
                   'auto-negotiate', 'comments', 'dhcp-ipsec',
                   'dhgrp', 'dst-addr-type', 'dst-end-ip',
                   'dst-end-ip6', 'dst-name', 'dst-name6',
                   'dst-port', 'dst-start-ip', 'dst-start-ip6',
                   'dst-subnet', 'dst-subnet6', 'encapsulation',
                   'keepalive', 'keylife-type', 'keylifekbs',
                   'keylifeseconds', 'l2tp', 'name',
                   'pfs', 'phase1name', 'proposal',
                   'protocol', 'replay', 'route-overlap',
                   'single-source', 'src-addr-type', 'src-end-ip',
                   'src-end-ip6', 'src-name', 'src-name6',
                   'src-port', 'src-start-ip', 'src-start-ip6',
                   'src-subnet', 'src-subnet6']
    dictionary = {}

    for attribute in option_list:
        if attribute in json and json[attribute] is not None:
            dictionary[attribute] = json[attribute]

    return dictionary


def flatten_multilists_attributes(data):
    multilist_attrs = []

    for attr in multilist_attrs:
        try:
            path = "data['" + "']['".join(elem for elem in attr) + "']"
            current_val = eval(path)
            flattened_val = ' '.join(elem for elem in current_val)
            exec(path + '= flattened_val')
        except BaseException:
            pass

    return data


def vpn_ipsec_phase2_interface(data, fos):
    vdom = data['vdom']
    vpn_ipsec_phase2_interface_data = data['vpn_ipsec_phase2_interface']
    flattened_data = flatten_multilists_attributes(vpn_ipsec_phase2_interface_data)
    filtered_data = filter_vpn_ipsec_phase2_interface_data(flattened_data)
    if vpn_ipsec_phase2_interface_data['state'] == "present":
        return fos.set('vpn.ipsec',
                       'phase2-interface',
                       data=filtered_data,
                       vdom=vdom)

    elif vpn_ipsec_phase2_interface_data['state'] == "absent":
        return fos.delete('vpn.ipsec',
                          'phase2-interface',
                          mkey=filtered_data['name'],
                          vdom=vdom)


def fortios_vpn_ipsec(data, fos):
    login(data)

    if data['vpn_ipsec_phase2_interface']:
        resp = vpn_ipsec_phase2_interface(data, fos)

    fos.logout()
    return not resp['status'] == "success", resp['status'] == "success", resp


def main():
    fields = {
        "host": {"required": True, "type": "str"},
        "username": {"required": True, "type": "str"},
        "password": {"required": False, "type": "str", "no_log": True},
        "vdom": {"required": False, "type": "str", "default": "root"},
        "https": {"required": False, "type": "bool", "default": True},
        "vpn_ipsec_phase2_interface": {
            "required": False, "type": "dict",
            "options": {
                "state": {"required": True, "type": "str",
                          "choices": ["present", "absent"]},
                "add-route": {"required": False, "type": "str",
                              "choices": ["phase1", "enable", "disable"]},
                "auto-discovery-forwarder": {"required": False, "type": "str",
                                             "choices": ["phase1", "enable", "disable"]},
                "auto-discovery-sender": {"required": False, "type": "str",
                                          "choices": ["phase1", "enable", "disable"]},
                "auto-negotiate": {"required": False, "type": "str",
                                   "choices": ["enable", "disable"]},
                "comments": {"required": False, "type": "str"},
                "dhcp-ipsec": {"required": False, "type": "str",
                               "choices": ["enable", "disable"]},
                "dhgrp": {"required": False, "type": "str",
                          "choices": ["1", "2", "5",
                                      "14", "15", "16",
                                      "17", "18", "19",
                                      "20", "21", "27",
                                      "28", "29", "30",
                                      "31"]},
                "dst-addr-type": {"required": False, "type": "str",
                                  "choices": ["subnet", "range", "ip",
                                              "name", "subnet6", "range6",
                                              "ip6", "name6"]},
                "dst-end-ip": {"required": False, "type": "str"},
                "dst-end-ip6": {"required": False, "type": "str"},
                "dst-name": {"required": False, "type": "str"},
                "dst-name6": {"required": False, "type": "str"},
                "dst-port": {"required": False, "type": "int"},
                "dst-start-ip": {"required": False, "type": "str"},
                "dst-start-ip6": {"required": False, "type": "str"},
                "dst-subnet": {"required": False, "type": "str"},
                "dst-subnet6": {"required": False, "type": "str"},
                "encapsulation": {"required": False, "type": "str",
                                  "choices": ["tunnel-mode", "transport-mode"]},
                "keepalive": {"required": False, "type": "str",
                              "choices": ["enable", "disable"]},
                "keylife-type": {"required": False, "type": "str",
                                 "choices": ["seconds", "kbs", "both"]},
                "keylifekbs": {"required": False, "type": "int"},
                "keylifeseconds": {"required": False, "type": "int"},
                "l2tp": {"required": False, "type": "str",
                         "choices": ["enable", "disable"]},
                "name": {"required": True, "type": "str"},
                "pfs": {"required": False, "type": "str",
                        "choices": ["enable", "disable"]},
                "phase1name": {"required": False, "type": "str"},
                "proposal": {"required": False, "type": "str",
                             "choices": ["null-md5", "null-sha1", "null-sha256",
                                         "null-sha384", "null-sha512", "des-null",
                                         "des-md5", "des-sha1", "des-sha256",
                                         "des-sha384", "des-sha512"]},
                "protocol": {"required": False, "type": "int"},
                "replay": {"required": False, "type": "str",
                           "choices": ["enable", "disable"]},
                "route-overlap": {"required": False, "type": "str",
                                  "choices": ["use-old", "use-new", "allow"]},
                "single-source": {"required": False, "type": "str",
                                  "choices": ["enable", "disable"]},
                "src-addr-type": {"required": False, "type": "str",
                                  "choices": ["subnet", "range", "ip",
                                              "name", "subnet6", "range6",
                                              "ip6", "name6"]},
                "src-end-ip": {"required": False, "type": "str"},
                "src-end-ip6": {"required": False, "type": "str"},
                "src-name": {"required": False, "type": "str"},
                "src-name6": {"required": False, "type": "str"},
                "src-port": {"required": False, "type": "int"},
                "src-start-ip": {"required": False, "type": "str"},
                "src-start-ip6": {"required": False, "type": "str"},
                "src-subnet": {"required": False, "type": "str"},
                "src-subnet6": {"required": False, "type": "str"}

            }
        }
    }

    module = AnsibleModule(argument_spec=fields,
                           supports_check_mode=False)
    try:
        from fortiosapi import FortiOSAPI
    except ImportError:
        module.fail_json(msg="fortiosapi module is required")

    global fos
    fos = FortiOSAPI()

    is_error, has_changed, result = fortios_vpn_ipsec(module.params, fos)

    if not is_error:
        module.exit_json(changed=has_changed, meta=result)
    else:
        module.fail_json(msg="Error in repo", meta=result)


if __name__ == '__main__':
    main()
